@@ -1,6 +1,6 @@
 /* ============================================================
 * QupZilla - WebKit based browser
-* Copyright (C) 2010-2012  David Rosca <nowrep@gmail.com>
+* Copyright (C) 2010-2013  David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -30,14 +30,14 @@
 #include "pluginproxy.h"
 #include "bookmarksmodel.h"
 #include "downloadmanager.h"
-#include "autofillmodel.h"
+#include "autofill.h"
 #include "adblockmanager.h"
 #include "desktopnotificationsfactory.h"
 #include "iconprovider.h"
 #include "qtwin.h"
 #include "mainapplication.h"
 #include "webhistoryinterface.h"
-#include "globalfunctions.h"
+#include "qztools.h"
 #include "profileupdater.h"
 #include "searchenginesmanager.h"
 #include "databasewriter.h"
@@ -50,11 +50,20 @@
 #include "useragentmanager.h"
 #include "restoremanager.h"
 #include "proxystyle.h"
+#include "qzregexp.h"
+#include "checkboxdialog.h"
+#include "registerqappassociation.h"
+#include "html5permissions/html5permissionsmanager.h"
+
+#ifdef USE_HUNSPELL
+#include "qtwebkit/spellcheck/speller.h"
+#endif
 
 #ifdef Q_OS_MAC
+#include "macmenureceiver.h"
 #include <QFileOpenEvent>
+#include <QMenu>
 #endif
-#include <QWebSecurityOrigin>
 #include <QNetworkDiskCache>
 #include <QDesktopServices>
 #include <QTranslator>
@@ -64,8 +73,8 @@
 #include <QTimer>
 #include <QDir>
 
-#if defined(PORTABLE_BUILD) && !defined(NO_SYSTEM_DATAPATH)
-#define NO_SYSTEM_DATAPATH
+#if QT_VERSION < 0x050000
+#include "qwebkitversion.h"
 #endif
 
 MainApplication::MainApplication(int &argc, char** argv)
@@ -85,16 +94,28 @@ MainApplication::MainApplication(int &argc, char** argv)
     , m_searchEnginesManager(0)
     , m_restoreManager(0)
     , m_proxyStyle(0)
+    , m_html5permissions(0)
+#ifdef USE_HUNSPELL
+    , m_speller(0)
+#endif
     , m_dbWriter(new DatabaseWriter(this))
     , m_uaManager(new UserAgentManager)
     , m_isPrivateSession(false)
+    , m_isPortable(false)
     , m_isClosing(false)
     , m_isStateChanged(false)
     , m_isRestoring(false)
     , m_startingAfterCrash(false)
     , m_databaseConnected(false)
+#if defined(Q_OS_WIN) && !defined(Q_OS_OS2)
+    , m_registerQAppAssociation(0)
+#endif
+#ifdef Q_OS_MAC
+    , m_macMenuReceiver(0)
+    , m_macDockMenu(0)
+#endif
 {
-#if defined(Q_WS_X11) && !defined(NO_SYSTEM_DATAPATH)
+#if defined(QZ_WS_X11) && !defined(NO_SYSTEM_DATAPATH)
     DATADIR = USE_DATADIR;
 #else
     DATADIR = qApp->applicationDirPath() + "/";
@@ -104,15 +125,6 @@ MainApplication::MainApplication(int &argc, char** argv)
     DATADIR.append(QLatin1String("../Resources/"));
 #endif
 
-#ifdef PORTABLE_BUILD
-    PROFILEDIR = DATADIR + "profiles/";
-#else
-    PROFILEDIR = QDir::homePath() + "/.qupzilla/";
-#endif
-
-    TRANSLATIONSDIR = DATADIR + "locale/";
-    THEMESDIR = DATADIR + "themes/";
-
     setWindowIcon(QIcon(":icons/exeicons/qupzilla-window.png"));
     bool noAddons = false;
     bool newInstance = false;
@@ -123,7 +135,7 @@ MainApplication::MainApplication(int &argc, char** argv)
     if (argc > 1) {
         CommandLineOptions cmd(argc);
 
-        foreach(const CommandLineOptions::ActionPair & pair, cmd.getActions()) {
+        foreach (const CommandLineOptions::ActionPair &pair, cmd.getActions()) {
             switch (pair.action) {
             case Qz::CL_StartWithoutAddons:
                 noAddons = true;
@@ -131,6 +143,9 @@ MainApplication::MainApplication(int &argc, char** argv)
             case Qz::CL_StartWithProfile:
                 startProfile = pair.text;
                 break;
+            case Qz::CL_StartPortable:
+                m_isPortable = true;
+                break;
             case Qz::CL_NewTab:
                 messages.append(QLatin1String("ACTION:NewTab"));
                 m_postLaunchActions.append(OpenNewTab);
@@ -138,6 +153,10 @@ MainApplication::MainApplication(int &argc, char** argv)
             case Qz::CL_NewWindow:
                 messages.append(QLatin1String("ACTION:NewWindow"));
                 break;
+            case Qz::CL_ToggleFullScreen:
+                messages.append(QLatin1String("ACTION:ToggleFullScreen"));
+                m_postLaunchActions.append(ToggleFullScreen);
+                break;
             case Qz::CL_ShowDownloadManager:
                 messages.append(QLatin1String("ACTION:ShowDownloadManager"));
                 m_postLaunchActions.append(OpenDownloadManager);
@@ -169,9 +188,28 @@ MainApplication::MainApplication(int &argc, char** argv)
         }
     }
 
+    if (isPortable()) {
+        std::cout << "QupZilla: Running in Portable Mode." << std::endl;
+        PROFILEDIR = DATADIR + "profiles/";
+    }
+    else {
+        bool confPathExists = QDir(QDir::homePath() + "/.config/qupzilla").exists();
+        bool homePathExists = QDir(QDir::homePath() + "/.qupzilla").exists();
+
+        if (homePathExists && !confPathExists) {
+            PROFILEDIR = QDir::homePath() + "/.qupzilla/";
+        }
+        else {
+            PROFILEDIR = QDir::homePath() + "/.config/qupzilla/";
+        }
+    }
+
+    TRANSLATIONSDIR = DATADIR + "locale/";
+    THEMESDIR = DATADIR + "themes/";
+
     // Don't start single application in private browsing
     if (!m_isPrivateSession) {
-        QString appId = "QupZillaWebBrowser";
+        QString appId = isPortable() ? "QupZillaWebBrowserPortable" : "QupZillaWebBrowser";
 
         if (newInstance) {
             if (startProfile.isEmpty() || startProfile == QLatin1String("default")) {
@@ -190,7 +228,7 @@ MainApplication::MainApplication(int &argc, char** argv)
     }
 
     if (isRunning()) {
-        foreach(const QString & message, messages) {
+        foreach (const QString &message, messages) {
             sendMessage(message);
         }
         m_isClosing = true;
@@ -207,6 +245,7 @@ MainApplication::MainApplication(int &argc, char** argv)
     setApplicationVersion(QupZilla::VERSION);
     setOrganizationDomain("qupzilla");
     QDesktopServices::setUrlHandler("http", this, "addNewTab");
+    QDesktopServices::setUrlHandler("ftp", this, "addNewTab");
 
     checkSettingsDir();
 
@@ -233,13 +272,12 @@ MainApplication::MainApplication(int &argc, char** argv)
     translateApp();
 
     QupZilla* qupzilla = new QupZilla(Qz::BW_FirstAppWindow, startUrl);
-    m_mainWindows.append(qupzilla);
+    m_mainWindows.prepend(qupzilla);
 
-    connect(qupzilla, SIGNAL(message(Qz::AppMessageType, bool)), this, SLOT(sendMessages(Qz::AppMessageType, bool)));
+    connect(qupzilla, SIGNAL(message(Qz::AppMessageType,bool)), this, SLOT(sendMessages(Qz::AppMessageType,bool)));
     connect(qupzilla, SIGNAL(startingCompleted()), this, SLOT(restoreCursor()));
 
     loadSettings();
-    networkManager()->loadCertificates();
 
     m_plugins = new PluginProxy;
 
@@ -258,8 +296,13 @@ MainApplication::MainApplication(int &argc, char** argv)
             new Updater(qupzilla);
         }
 
+        backupSavedSessions();
+
         if (m_startingAfterCrash || afterLaunch == 3) {
             m_restoreManager = new RestoreManager(m_activeProfil + "session.dat");
+            if (!m_restoreManager->isValid()) {
+                destroyRestoreManager();
+            }
         }
     }
 
@@ -276,7 +319,11 @@ void MainApplication::postLaunch()
     }
 
     if (m_postLaunchActions.contains(OpenNewTab)) {
-        getWindow()->tabWidget()->addView(QUrl(), Qz::NT_SelectedTabAtTheEnd);
+        getWindow()->tabWidget()->addView(QUrl(), Qz::NT_SelectedNewEmptyTab);
+    }
+
+    if (m_postLaunchActions.contains(ToggleFullScreen)) {
+        getWindow()->toggleFullScreen();
     }
 
     AutoSaver* saver = new AutoSaver();
@@ -287,6 +334,15 @@ void MainApplication::postLaunch()
 
     connect(this, SIGNAL(messageReceived(QString)), this, SLOT(receiveAppMessage(QString)));
     connect(this, SIGNAL(aboutToQuit()), this, SLOT(saveSettings()));
+
+    if (!isPortable()) {
+        Settings settings;
+        bool alwaysCheckDefaultBrowser = settings.value("Web-Browser-Settings/CheckDefaultBrowser", DEFAULT_CHECK_DEFAULTBROWSER).toBool();
+        if (alwaysCheckDefaultBrowser) {
+            alwaysCheckDefaultBrowser = checkDefaultWebBrowser();
+            settings.setValue("Web-Browser-Settings/CheckDefaultBrowser", alwaysCheckDefaultBrowser);
+        }
+    }
 }
 
 void MainApplication::loadSettings()
@@ -295,81 +351,48 @@ void MainApplication::loadSettings()
     settings.beginGroup("Themes");
     QString activeTheme = settings.value("activeTheme", DEFAULT_THEME_NAME).toString();
     settings.endGroup();
-    m_activeThemePath = THEMESDIR + activeTheme + "/";
-    QFile cssFile(m_activeThemePath + "main.css");
-    cssFile.open(QFile::ReadOnly);
-    QString css = cssFile.readAll();
-    cssFile.close();
-#ifdef Q_WS_X11
-    if (QFile(m_activeThemePath + "linux.css").exists()) {
-        cssFile.setFileName(m_activeThemePath + "linux.css");
-        cssFile.open(QFile::ReadOnly);
-        css.append(cssFile.readAll());
-        cssFile.close();
-    }
-#endif
-#ifdef Q_OS_MAC
-    if (QFile(m_activeThemePath + "mac.css").exists()) {
-        cssFile.setFileName(m_activeThemePath + "mac.css");
-        cssFile.open(QFile::ReadOnly);
-        css.append(cssFile.readAll());
-        cssFile.close();
-    }
-#endif
-#if defined(Q_OS_WIN) || defined(Q_OS_OS2)
-    if (QFile(m_activeThemePath + "windows.css").exists()) {
-        cssFile.setFileName(m_activeThemePath + "windows.css");
-        cssFile.open(QFile::ReadOnly);
-        css.append(cssFile.readAll());
-        cssFile.close();
-    }
-#endif
 
-    //RTL Support
-    //loading 'rtl.css' when layout is right to left!
-    if (isRightToLeft() && QFile(m_activeThemePath + "rtl.css").exists()) {
-        cssFile.setFileName(m_activeThemePath + "rtl.css");
-        cssFile.open(QFile::ReadOnly);
-        css.append(cssFile.readAll());
-        cssFile.close();
-    }
-
-    QString relativePath = QDir::current().relativeFilePath(m_activeThemePath);
-    css.replace(QRegExp("url\\s*\\(\\s*([^\\*:\\);]+)\\s*\\)", Qt::CaseSensitive, QRegExp::RegExp2),
-                QString("url(%1\\1)").arg(relativePath + "/"));
-    setStyleSheet(css);
+    loadTheme(activeTheme);
 
+    // Create global QWebSettings object
     webSettings();
 
-    //Web browsing settings
+    // Web browsing settings
     settings.beginGroup("Web-Browser-Settings");
 
     if (!m_isPrivateSession) {
         m_websettings->enablePersistentStorage(m_activeProfil);
         m_websettings->setAttribute(QWebSettings::LocalStorageEnabled, settings.value("HTML5StorageEnabled", true).toBool());
     }
+
     m_websettings->setAttribute(QWebSettings::DeveloperExtrasEnabled, true);
     m_websettings->setAttribute(QWebSettings::PluginsEnabled, settings.value("allowFlash", true).toBool());
     m_websettings->setAttribute(QWebSettings::JavascriptEnabled, settings.value("allowJavaScript", true).toBool());
     m_websettings->setAttribute(QWebSettings::JavascriptCanOpenWindows, settings.value("allowJavaScriptOpenWindow", false).toBool());
     m_websettings->setAttribute(QWebSettings::JavaEnabled, settings.value("allowJava", true).toBool());
     m_websettings->setAttribute(QWebSettings::DnsPrefetchEnabled, settings.value("DNS-Prefetch", false).toBool());
-    m_websettings->setAttribute(QWebSettings::JavascriptCanAccessClipboard, settings.value("JavaScriptCanAccessClipboard", true).toBool());
+    m_websettings->setAttribute(QWebSettings::JavascriptCanAccessClipboard, settings.value("allowJavaScriptAccessClipboard", true).toBool());
     m_websettings->setAttribute(QWebSettings::LinksIncludedInFocusChain, settings.value("IncludeLinkInFocusChain", false).toBool());
     m_websettings->setAttribute(QWebSettings::ZoomTextOnly, settings.value("zoomTextOnly", false).toBool());
     m_websettings->setAttribute(QWebSettings::PrintElementBackgrounds, settings.value("PrintElementBackground", true).toBool());
     m_websettings->setAttribute(QWebSettings::XSSAuditingEnabled, settings.value("XSSAuditing", false).toBool());
     m_websettings->setMaximumPagesInCache(settings.value("maximumCachedPages", 3).toInt());
     m_websettings->setDefaultTextEncoding(settings.value("DefaultEncoding", m_websettings->defaultTextEncoding()).toString());
+    m_websettings->setAttribute(QWebSettings::SpatialNavigationEnabled, settings.value("SpatialNavigation", false).toBool());
+
+#if QTWEBKIT_FROM_2_3
+    m_websettings->setAttribute(QWebSettings::CaretBrowsingEnabled, settings.value("CaretBrowsing", false).toBool());
+    m_websettings->setAttribute(QWebSettings::ScrollAnimatorEnabled, settings.value("AnimateScrolling", true).toBool());
+#endif
 
 #ifdef USE_WEBGL
     m_websettings->setAttribute(QWebSettings::WebGLEnabled, true);
     m_websettings->setAttribute(QWebSettings::AcceleratedCompositingEnabled, true);
 #endif
 
-#if (QTWEBKIT_VERSION >= QTWEBKIT_VERSION_CHECK(2, 2, 0))
+#if QTWEBKIT_FROM_2_2
     m_websettings->setAttribute(QWebSettings::HyperlinkAuditingEnabled, true);
-    m_websettings->setAttribute(QWebSettings::JavascriptCanCloseWindows, true);
+    m_websettings->setAttribute(QWebSettings::JavascriptCanCloseWindows, settings.value("allowJavaScriptCloseWindow", false).toBool());
 #endif
 
     setWheelScrollLines(settings.value("wheelScrollLines", wheelScrollLines()).toInt());
@@ -392,9 +415,6 @@ void MainApplication::loadSettings()
     m_websettings->setWebGraphic(QWebSettings::DefaultFrameIconGraphic, qIconProvider->emptyWebIcon().pixmap(16, 16));
     m_websettings->setWebGraphic(QWebSettings::MissingImageGraphic, QPixmap());
 
-    // Allows to load files from qrc: scheme in qupzilla: pages
-    QWebSecurityOrigin::addLocalScheme("qupzilla");
-
     if (m_isPrivateSession) {
         m_websettings->setAttribute(QWebSettings::PrivateBrowsingEnabled, true);
         history()->setSaving(false);
@@ -410,7 +430,8 @@ void MainApplication::loadSettings()
 
 void MainApplication::reloadSettings()
 {
-    loadSettings();
+    QTimer::singleShot(0, this, SLOT(loadSettings()));
+
     emit message(Qz::AM_ReloadSettings, true);
 }
 
@@ -472,6 +493,61 @@ QList<QupZilla*> MainApplication::mainWindows()
     return list;
 }
 
+bool MainApplication::isClosing() const
+{
+    return m_isClosing;
+}
+
+bool MainApplication::isRestoring() const
+{
+    return m_isRestoring;
+}
+
+bool MainApplication::isPrivateSession() const
+{
+    return m_isPrivateSession;
+}
+
+bool MainApplication::isPortable() const
+{
+#ifdef PORTABLE_BUILD
+    return true;
+#else
+    return m_isPortable;
+#endif
+}
+
+bool MainApplication::isStartingAfterCrash() const
+{
+    return m_startingAfterCrash;
+}
+
+int MainApplication::windowCount() const
+{
+    return m_mainWindows.count();
+}
+
+QString MainApplication::currentLanguageFile() const
+{
+    return m_activeLanguage;
+}
+
+QString MainApplication::currentLanguage() const
+{
+    QString lang = m_activeLanguage;
+
+    if (lang.isEmpty()) {
+        return "en_US";
+    }
+
+    return lang.left(lang.length() - 3);
+}
+
+QString MainApplication::currentProfilePath() const
+{
+    return m_activeProfil;
+}
+
 void MainApplication::sendMessages(Qz::AppMessageType mes, bool state)
 {
     emit message(mes, state);
@@ -483,12 +559,12 @@ void MainApplication::receiveAppMessage(QString message)
     QUrl actUrl;
 
     if (message.startsWith(QLatin1String("URL:"))) {
-        QUrl url = QUrl::fromUserInput(message.mid(4));
+        const QUrl url = QUrl::fromUserInput(message.mid(4));
         addNewTab(url);
         actWin = getWindow();
     }
     else if (message.startsWith(QLatin1String("ACTION:"))) {
-        QString text = message.mid(7);
+        const QString text = message.mid(7);
         if (text == QLatin1String("NewTab")) {
             addNewTab();
         }
@@ -499,6 +575,10 @@ void MainApplication::receiveAppMessage(QString message)
             downManager()->show();
             actWin = downManager();
         }
+        else if (text == QLatin1String("ToggleFullScreen") && actWin) {
+            QupZilla* qz = static_cast<QupZilla*>(actWin);
+            qz->toggleFullScreen();
+        }
         else if (text.startsWith(QLatin1String("OpenUrlInCurrentTab"))) {
             actUrl = QUrl::fromUserInput(text.mid(19));
         }
@@ -508,18 +588,21 @@ void MainApplication::receiveAppMessage(QString message)
         }
     }
 
-    if (!actWin && !isClosing()) { // It can only occur if download manager window was still open
-        makeNewWindow(Qz::BW_NewWindow, actUrl);
+    if (!actWin) {
+        if (!isClosing()) {
+            // It can only occur if download manager window was still opened
+            makeNewWindow(Qz::BW_NewWindow, actUrl);
+        }
         return;
     }
 
-    QupZilla* qz = qobject_cast<QupZilla*>(actWin);
-
     actWin->setWindowState(actWin->windowState() & ~Qt::WindowMinimized);
     actWin->raise();
     actWin->activateWindow();
     actWin->setFocus();
 
+    QupZilla* qz = qobject_cast<QupZilla*>(actWin);
+
     if (qz && !actUrl.isEmpty()) {
         qz->loadAddress(actUrl);
     }
@@ -530,22 +613,42 @@ void MainApplication::addNewTab(const QUrl &url)
     if (!getWindow()) {
         return;
     }
-    getWindow()->tabWidget()->addView(url, Qz::NT_SelectedTabAtTheEnd);
+
+    getWindow()->tabWidget()->addView(url, url.isEmpty() ? Qz::NT_SelectedNewEmptyTab : Qz::NT_SelectedTabAtTheEnd);
 }
 
 QupZilla* MainApplication::makeNewWindow(Qz::BrowserWindow type, const QUrl &startUrl)
 {
-    if (m_mainWindows.count() == 0) {
+    if (m_mainWindows.count() == 0 && type != Qz::BW_MacFirstWindow) {
         type = Qz::BW_FirstAppWindow;
     }
 
     QupZilla* newWindow = new QupZilla(type, startUrl);
-    m_mainWindows.append(newWindow);
+    m_mainWindows.prepend(newWindow);
 
     return newWindow;
 }
 
 #ifdef Q_OS_MAC
+extern void qt_mac_set_dock_menu(QMenu* menu);
+
+QMenu* MainApplication::macDockMenu()
+{
+    if (!m_macDockMenu) {
+        m_macDockMenu = new QMenu(0);
+        qt_mac_set_dock_menu(m_macDockMenu);
+    }
+    return m_macDockMenu;
+}
+
+MacMenuReceiver* MainApplication::macMenuReceiver()
+{
+    if (!m_macMenuReceiver) {
+        m_macMenuReceiver = new MacMenuReceiver(this);
+    }
+    return m_macMenuReceiver;
+}
+
 bool MainApplication::event(QEvent* e)
 {
     switch (e->type()) {
@@ -591,10 +694,106 @@ void MainApplication::connectDatabase()
     m_databaseConnected = true;
 }
 
+void MainApplication::loadTheme(const QString &name)
+{
+    // Themes are loaded from the following directories:
+    //  1. Directory "themes" in user profile
+    //  2. Directory "themes" in root profile directory
+    //  3. System data path
+    //       > /usr/share/qupzilla/themes on Linux
+    //       > $EXECUTABLE_DIR/themes on Windows
+
+    QStringList themePaths;
+    themePaths << m_activeProfil + "themes/"
+               << PROFILEDIR + "themes/"
+               << THEMESDIR;
+
+    foreach (const QString &path, themePaths) {
+        const QString theme = path + name + "/";
+        if (QFile::exists(theme + "main.css")) {
+            m_activeThemePath = theme;
+            break;
+        }
+    }
+
+    if (m_activeThemePath.isEmpty()) {
+        qWarning("Cannot load theme '%s'!", qPrintable(name));
+        m_activeThemePath = THEMESDIR + DEFAULT_THEME_NAME + "/";
+    }
+
+    QFile cssFile(m_activeThemePath + "main.css");
+    cssFile.open(QFile::ReadOnly);
+    QString css = cssFile.readAll();
+    cssFile.close();
+
+    /*
+     * #id[style=QtStyle] (QtStyle = QMacStyle, QWindowsVistaStyle, QGtkStyle, ...)
+     * should be enough instead of loading special stylesheets
+     */
+#ifdef QZ_WS_X11
+    if (QFile(m_activeThemePath + "linux.css").exists()) {
+        cssFile.setFileName(m_activeThemePath + "linux.css");
+        cssFile.open(QFile::ReadOnly);
+        css.append(cssFile.readAll());
+        cssFile.close();
+    }
+#endif
+
+#ifdef Q_OS_MAC
+    if (QFile(m_activeThemePath + "mac.css").exists()) {
+        cssFile.setFileName(m_activeThemePath + "mac.css");
+        cssFile.open(QFile::ReadOnly);
+        css.append(cssFile.readAll());
+        cssFile.close();
+    }
+#endif
+
+#if defined(Q_OS_WIN) || defined(Q_OS_OS2)
+    if (QFile(m_activeThemePath + "windows.css").exists()) {
+        cssFile.setFileName(m_activeThemePath + "windows.css");
+        cssFile.open(QFile::ReadOnly);
+        css.append(cssFile.readAll());
+        cssFile.close();
+    }
+#endif
+
+    // RTL Support
+    // Loading 'rtl.css' when layout is right to left!
+    if (isRightToLeft() && QFile(m_activeThemePath + "rtl.css").exists()) {
+        cssFile.setFileName(m_activeThemePath + "rtl.css");
+        cssFile.open(QFile::ReadOnly);
+        css.append(cssFile.readAll());
+        cssFile.close();
+    }
+
+    QString relativePath = QDir::current().relativeFilePath(m_activeThemePath);
+    css.replace(QzRegExp("url\\s*\\(\\s*([^\\*:\\);]+)\\s*\\)", Qt::CaseSensitive),
+                QString("url(%1\\1)").arg(relativePath + "/"));
+
+    setStyleSheet(css);
+}
+
 void MainApplication::translateApp()
 {
     Settings settings;
-    const QString &file = settings.value("Language/language", QLocale::system().name()).toString();
+    QString file = settings.value("Language/language", QLocale::system().name()).toString();
+
+    if (!file.isEmpty() && !file.endsWith(QLatin1String(".qm"))) {
+        file.append(".qm");
+    }
+
+    // If "xx_yy" translation doesn't exists, try to use "xx*" translation
+    // It can only happen when Language is chosen from system locale
+
+    if (!file.isEmpty() && !QFile(TRANSLATIONSDIR + QLatin1String("/") + file).exists()) {
+        QDir translationsDir(TRANSLATIONSDIR);
+        QString lang = file.left(2) + QLatin1String("*.qm");
+
+        const QStringList translations = translationsDir.entryList(QStringList(lang));
+
+        // If no translation can be found, default English will be used
+        file = translations.isEmpty() ? QString() : translations.first();
+    }
 
     QTranslator* app = new QTranslator(this);
     app->load(file, TRANSLATIONSDIR);
@@ -608,6 +807,27 @@ void MainApplication::translateApp()
     installTranslator(sys);
 }
 
+void MainApplication::backupSavedSessions()
+{
+    // session.dat      - current
+    // session.dat.old  - first backup
+    // session.dat.old1 - second backup
+
+    const QString sessionFile = m_activeProfil + "session.dat";
+
+    if (!QFile::exists(sessionFile)) {
+        return;
+    }
+
+    if (QFile::exists(sessionFile + ".old")) {
+        QFile::remove(sessionFile + ".old1");
+        QFile::copy(sessionFile + ".old", sessionFile + ".old1");
+    }
+
+    QFile::remove(sessionFile + ".old");
+    QFile::copy(sessionFile, sessionFile + ".old");
+}
+
 void MainApplication::quitApplication()
 {
     if (m_downloadManager && !m_downloadManager->canClose()) {
@@ -628,6 +848,10 @@ void MainApplication::quitApplication()
     //
     //  * this can occur on Mac OS (see #157)
 
+    if (!m_isPrivateSession) {
+        removeLockFile();
+    }
+
     quit();
 }
 
@@ -646,14 +870,10 @@ void MainApplication::saveSettings()
     settings.endGroup();
 
     settings.beginGroup("Web-Browser-Settings");
-    bool deleteCookies = settings.value("deleteCookiesOnClose", false).toBool();
     bool deleteHistory = settings.value("deleteHistoryOnClose", false).toBool();
     bool deleteHtml5Storage = settings.value("deleteHTML5StorageOnClose", false).toBool();
     settings.endGroup();
 
-    if (deleteCookies) {
-        m_cookiejar->clearCookies();
-    }
     if (deleteHistory) {
         m_historymodel->clearHistory();
     }
@@ -665,7 +885,9 @@ void MainApplication::saveSettings()
     m_networkmanager->saveCertificates();
     m_plugins->shutdown();
     qIconProvider->saveIconsToDatabase();
+    clearTempPath();
 
+    qzSettings->saveSettings();
     AdBlockManager::instance()->save();
     QFile::remove(currentProfilePath() + "WebpageIcons.db");
     Settings::syncSettings();
@@ -706,7 +928,7 @@ QWebSettings* MainApplication::webSettings()
 NetworkManager* MainApplication::networkManager()
 {
     if (!m_networkmanager) {
-        m_networkmanager = new NetworkManager(getWindow());
+        m_networkmanager = new NetworkManager(this);
     }
     return m_networkmanager;
 }
@@ -749,10 +971,10 @@ DownloadManager* MainApplication::downManager()
     return m_downloadManager;
 }
 
-AutoFillModel* MainApplication::autoFill()
+AutoFill* MainApplication::autoFill()
 {
     if (!m_autofill) {
-        m_autofill = new AutoFillModel(getWindow());
+        m_autofill = new AutoFill(this);
     }
     return m_autofill;
 }
@@ -768,8 +990,13 @@ SearchEnginesManager* MainApplication::searchEnginesManager()
 QNetworkDiskCache* MainApplication::networkCache()
 {
     if (!m_networkCache) {
+        Settings settings;
+        const QString basePath = settings.value("Web-Browser-Settings/CachePath",
+                                                QString("%1networkcache/").arg(m_activeProfil)).toString();
+
+        const QString cachePath = QString("%1/%2-Qt%3/").arg(basePath, qWebKitVersion(), qVersion());
         m_networkCache = new QNetworkDiskCache(this);
-        m_networkCache->setCacheDirectory(m_activeProfil + "/networkcache");
+        m_networkCache->setCacheDirectory(cachePath);
     }
 
     return m_networkCache;
@@ -783,11 +1010,31 @@ DesktopNotificationsFactory* MainApplication::desktopNotifications()
     return m_desktopNotifications;
 }
 
+HTML5PermissionsManager* MainApplication::html5permissions()
+{
+    if (!m_html5permissions) {
+        m_html5permissions = new HTML5PermissionsManager(this);
+    }
+    return m_html5permissions;
+}
+
+#ifdef USE_HUNSPELL
+Speller* MainApplication::speller()
+{
+    if (!m_speller) {
+        m_speller = new Speller(this);
+    }
+    return m_speller;
+}
+#endif
+
 void MainApplication::startPrivateBrowsing()
 {
     QStringList args;
-    foreach(const QString & arg, arguments()) {
-        if (arg.startsWith(QLatin1Char('-'))) {
+    foreach (const QString &arg, arguments()) {
+        if (arg.startsWith(QLatin1Char('-')) &&
+                arg != QLatin1String("--private-browsing") &&
+                arg != QLatin1String("-pb")) {
             args.append(arg);
         }
     }
@@ -807,9 +1054,69 @@ void MainApplication::reloadUserStyleSheet()
     settings.endGroup();
 }
 
+bool MainApplication::checkDefaultWebBrowser()
+{
+#if defined(Q_OS_WIN) && !defined(Q_OS_OS2)
+    bool showAgain = true;
+    if (!associationManager()->isDefaultForAllCapabilities()) {
+        CheckBoxDialog dialog(QDialogButtonBox::Yes | QDialogButtonBox::No, getWindow());
+        dialog.setText(tr("QupZilla is not currently your default browser. Would you like to make it your default browser?"));
+        dialog.setCheckBoxText(tr("Always perform this check when starting QupZilla."));
+        dialog.setDefaultCheckState(Qt::Checked);
+        dialog.setWindowTitle(tr("Default Browser"));
+        dialog.setIcon(qIconProvider->standardIcon(QStyle::SP_MessageBoxWarning));
+
+        if (dialog.exec() == QDialog::Accepted) {
+            associationManager()->registerAllAssociation();
+        }
+
+        showAgain = dialog.isChecked();
+    }
+
+    return showAgain;
+#else
+    return false;
+#endif
+}
+
+#if defined(Q_OS_WIN) && !defined(Q_OS_OS2)
+RegisterQAppAssociation* MainApplication::associationManager()
+{
+    if (!m_registerQAppAssociation) {
+        QString desc = tr("QupZilla is a new, fast and secure open-source WWW browser. QupZilla is licensed under GPL version 3 or (at your option) any later version. It is based on WebKit core and Qt Framework.");
+        QString fileIconPath = QApplication::applicationFilePath() + ",1";
+        QString appIconPath = QApplication::applicationFilePath() + ",0";
+        m_registerQAppAssociation = new RegisterQAppAssociation("QupZilla", QApplication::applicationFilePath(), appIconPath, desc, this);
+        m_registerQAppAssociation->addCapability(".html", "QupZilla.HTML", "HTML File", fileIconPath, RegisterQAppAssociation::FileAssociation);
+        m_registerQAppAssociation->addCapability(".htm", "QupZilla.HTM", "HTM File", fileIconPath, RegisterQAppAssociation::FileAssociation);
+        m_registerQAppAssociation->addCapability("http", "QupZilla.HTTP", "URL:HyperText Transfer Protocol", appIconPath, RegisterQAppAssociation::UrlAssociation);
+        m_registerQAppAssociation->addCapability("https", "QupZilla.HTTPS", "URL:HyperText Transfer Protocol with Privacy", appIconPath, RegisterQAppAssociation::UrlAssociation);
+        m_registerQAppAssociation->addCapability("ftp", "QupZilla.FTP", "URL:File Transfer Protocol", appIconPath, RegisterQAppAssociation::UrlAssociation);
+    }
+    return m_registerQAppAssociation;
+}
+#endif
+
 QUrl MainApplication::userStyleSheet(const QString &filePath) const
 {
-    QString userStyle = AdBlockManager::instance()->elementHidingRules() + "{ display:none !important;}";
+    QString userStyle;
+
+#ifndef QZ_WS_X11
+    // Don't grey out selection on losing focus (to prevent graying out found text)
+    QString highlightColor;
+    QString highlightedTextColor;
+#ifdef Q_OS_MAC
+    highlightColor = QLatin1String("#b6d6fc");
+    highlightedTextColor = QLatin1String("#000");
+#else
+    QPalette pal = style()->standardPalette();
+    highlightColor = pal.color(QPalette::Highlight).name();
+    highlightedTextColor = pal.color(QPalette::HighlightedText).name();
+#endif
+    userStyle += QString("::selection {background: %1; color: %2;} ").arg(highlightColor, highlightedTextColor);
+#endif
+
+    userStyle += AdBlockManager::instance()->elementHidingRules();
 
     QFile file(filePath);
     if (!filePath.isEmpty() && file.open(QFile::ReadOnly)) {
@@ -819,8 +1126,8 @@ QUrl MainApplication::userStyleSheet(const QString &filePath) const
         file.close();
     }
 
-    const QString &encodedStyle = userStyle.toAscii().toBase64();
-    const QString &dataString = QString("data:text/css;charset=utf-8;base64,%1").arg(encodedStyle);
+    const QString encodedStyle = userStyle.toLatin1().toBase64();
+    const QString dataString = QString("data:text/css;charset=utf-8;base64,%1").arg(encodedStyle);
 
     return QUrl(dataString);
 }
@@ -844,9 +1151,6 @@ void MainApplication::aboutToCloseWindow(QupZilla* window)
     m_mainWindows.removeOne(window);
 }
 
-// Version of session.dat file
-static const int sessionVersion = 0x0003;
-
 bool MainApplication::saveStateSlot()
 {
     if (m_isPrivateSession || m_isRestoring || m_mainWindows.count() == 0 || m_restoreManager) {
@@ -857,7 +1161,7 @@ bool MainApplication::saveStateSlot()
     file.open(QIODevice::WriteOnly);
     QDataStream stream(&file);
 
-    stream << sessionVersion;
+    stream << Qz::sessionVersion;
     stream << m_mainWindows.count();
 
     for (int i = 0; i < m_mainWindows.count(); i++) {
@@ -904,16 +1208,27 @@ bool MainApplication::restoreStateSlot(QupZilla* window, RestoreData recoveryDat
         // Instead create new one and restore pinned tabs there
 
         QupZilla* newWin = makeNewWindow(Qz::BW_OtherRestoredWindow);
-        newWin->tabWidget()->restorePinnedTabs();
-        newWin->restoreWindowState(recoveryData.takeFirst());
+        newWin->restoreWindowState(recoveryData.first());
+        recoveryData.remove(0);
     }
     else {
-        window->restoreWindowState(recoveryData.takeFirst());
+        // QTabWidget::count() - count of tabs is not updated after closing
+        // recovery tab ...
+        // update: it seems with ComboTabBar QTabWidget::count() is updated,
+        // we add pinnedTabCounts to currentTab!
+        int tabCount = window->tabWidget()->pinnedTabsCount();
+        RestoreManager::WindowData data = recoveryData.first();
+        data.currentTab += tabCount;
+        recoveryData.remove(0);
+
+        window->restoreWindowState(data);
     }
 
-    foreach(const RestoreManager::WindowData & data, recoveryData) {
+    foreach (const RestoreManager::WindowData &data, recoveryData) {
         QupZilla* window = makeNewWindow(Qz::BW_OtherRestoredWindow);
         window->restoreWindowState(data);
+        // for correct geometry calculation in QupZilla::setupUi()
+        mApp->processEvents();
     }
 
     destroyRestoreManager();
@@ -927,7 +1242,7 @@ bool MainApplication::checkSettingsDir()
     /*
     $HOMEDIR
         |
-    .qupzilla/
+    .config/qupzilla/
         |
     profiles/-----------
         |              |
@@ -951,7 +1266,7 @@ bool MainApplication::checkSettingsDir()
     dir.mkdir("profiles");
     dir.cd("profiles");
 
-    //.qupzilla/profiles
+    //.config/qupzilla/profiles
     QFile(PROFILEDIR + "profiles/profiles.ini").remove();
     QFile(":data/profiles.ini").copy(PROFILEDIR + "profiles/profiles.ini");
     QFile(PROFILEDIR + "profiles/profiles.ini").setPermissions(QFile::ReadUser | QFile::WriteUser);
@@ -959,7 +1274,7 @@ bool MainApplication::checkSettingsDir()
     dir.mkdir("default");
     dir.cd("default");
 
-    //.qupzilla/profiles/default
+    //.config/qupzilla/profiles/default
     QFile(PROFILEDIR + "profiles/default/browsedata.db").remove();
     QFile(":data/browsedata.db").copy(PROFILEDIR + "profiles/default/browsedata.db");
     QFile(PROFILEDIR + "profiles/default/browsedata.db").setPermissions(QFile::ReadUser | QFile::WriteUser);
@@ -977,6 +1292,11 @@ void MainApplication::destroyRestoreManager()
     m_restoreManager = 0;
 }
 
+ProxyStyle* MainApplication::proxyStyle() const
+{
+    return m_proxyStyle;
+}
+
 void MainApplication::setProxyStyle(ProxyStyle* style)
 {
     m_proxyStyle = style;
@@ -989,7 +1309,38 @@ QString MainApplication::currentStyle() const
     return m_proxyStyle->baseStyle()->objectName();
 }
 
+void MainApplication::clearTempPath()
+{
+    QString path = PROFILEDIR + "tmp/";
+    QDir dir(path);
+
+    if (dir.exists()) {
+        QzTools::removeDir(path);
+    }
+}
+
+QString MainApplication::tempPath() const
+{
+    QString path = PROFILEDIR + "tmp/";
+    QDir dir(path);
+    if (!dir.exists()) {
+#ifdef QZ_WS_X11
+        // Symlink it to standard temporary path /tmp
+        QDir().mkpath(QDir::tempPath() + "/qupzilla/tmp");
+        QFile::remove(PROFILEDIR + "tmp");
+        QFile::link(QDir::tempPath() + "/qupzilla/tmp/", PROFILEDIR + "tmp");
+#else
+        dir.mkdir(path);
+#endif
+    }
+
+    return path;
+}
+
 MainApplication::~MainApplication()
 {
     delete m_uaManager;
+#ifdef Q_OS_MAC
+    delete m_macDockMenu;
+#endif
 }
