@@ -1,6 +1,6 @@
 /* ============================================================
 * QupZilla - WebKit based browser
-* Copyright (C) 2010-2012  David Rosca <nowrep@gmail.com>
+* Copyright (C) 2010-2014  David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -21,12 +21,9 @@
 #include "rssmanager.h"
 #include "mainapplication.h"
 #include "clickablelabel.h"
-#include "siteinfowidget.h"
-#include "rsswidget.h"
 #include "webpage.h"
 #include "tabwidget.h"
 #include "bookmarkicon.h"
-#include "bookmarkswidget.h"
 #include "progressbar.h"
 #include "statusbarmessage.h"
 #include "toolbutton.h"
@@ -35,67 +32,89 @@
 #include "goicon.h"
 #include "rssicon.h"
 #include "downicon.h"
-#include "globalfunctions.h"
+#include "qztools.h"
 #include "iconprovider.h"
 #include "qzsettings.h"
+#include "colors.h"
+#include "autofillicon.h"
 
+#include <QMimeData>
 #include <QClipboard>
 #include <QTimer>
 #include <QContextMenuEvent>
+#include <QAction>
+#include <QMenu>
 
 LocationBar::LocationBar(QupZilla* mainClass)
     : LineEdit(mainClass)
     , p_QupZilla(mainClass)
     , m_webView(0)
-    , m_menu(new QMenu(this))
     , m_pasteAndGoAction(0)
     , m_clearAction(0)
+    , m_rssIconVisible(false)
     , m_holdingAlt(false)
     , m_loadProgress(0)
-    , m_loadFinished(true)
+    , m_progressVisible(false)
+    , m_forcePaintEvent(false)
+    , m_inlineCompletionVisible(false)
+    , m_popupClosed(false)
 {
     setObjectName("locationbar");
     setDragEnabled(true);
 
-    m_bookmarkIcon = new BookmarkIcon(p_QupZilla);
+    m_bookmarkIcon = new BookmarkIcon(this);
     m_goIcon = new GoIcon(this);
     m_rssIcon = new RssIcon(this);
-    m_rssIcon->setToolTip(tr("Add RSS from this page..."));
-    m_siteIcon = new SiteIcon(this);
+    m_siteIcon = new SiteIcon(p_QupZilla, this);
+    m_autofillIcon = new AutoFillIcon(this);
     DownIcon* down = new DownIcon(this);
 
-    ////RTL Support
-    ////if we don't add 'm_siteIcon' by following code, then we should use suitable padding-left value
-    //// but then, when typing RTL text the layout dynamically changed and within RTL layout direction
-    //// padding-left is equivalent to padding-right and vice versa, and because style sheet is
-    //// not changed dynamically this create padding problems.
+    // RTL Support
+    // if we don't add 'm_siteIcon' by following code, then we should use suitable padding-left value
+    // but then, when typing RTL text the layout dynamically changed and within RTL layout direction
+    // padding-left is equivalent to padding-right and vice versa, and because style sheet is
+    // not changed dynamically this create padding problems.
     addWidget(m_siteIcon, LineEdit::LeftSide);
 
-    addWidget(m_goIcon, LineEdit::RightSide);
+    addWidget(m_autofillIcon, LineEdit::RightSide);
     addWidget(m_bookmarkIcon, LineEdit::RightSide);
     addWidget(m_rssIcon, LineEdit::RightSide);
+    addWidget(m_goIcon, LineEdit::RightSide);
     addWidget(down, LineEdit::RightSide);
 
     m_completer.setLocationBar(this);
     connect(&m_completer, SIGNAL(showCompletion(QString)), this, SLOT(showCompletion(QString)));
     connect(&m_completer, SIGNAL(completionActivated()), this, SLOT(urlEnter()));
+    connect(&m_completer, SIGNAL(popupClosed()), this, SLOT(completionPopupClosed()));
 
     connect(this, SIGNAL(textEdited(QString)), this, SLOT(textEdit()));
-    connect(m_siteIcon, SIGNAL(clicked()), this, SLOT(showSiteInfo()));
     connect(m_goIcon, SIGNAL(clicked(QPoint)), this, SLOT(urlEnter()));
-    connect(m_rssIcon, SIGNAL(clicked(QPoint)), this, SLOT(rssIconClicked()));
-    connect(m_bookmarkIcon, SIGNAL(clicked(QPoint)), this, SLOT(bookmarkIconClicked()));
-    connect(down, SIGNAL(clicked(QPoint)), this, SLOT(showMostVisited()));
+    connect(down, SIGNAL(clicked(QPoint)), &m_completer, SLOT(showMostVisited()));
     connect(mApp->searchEnginesManager(), SIGNAL(activeEngineChanged()), this, SLOT(updatePlaceHolderText()));
+    connect(mApp->searchEnginesManager(), SIGNAL(defaultEngineChanged()), this, SLOT(updatePlaceHolderText()));
+    connect(mApp, SIGNAL(message(Qz::AppMessageType,bool)), SLOT(onMessage(Qz::AppMessageType,bool)));
+
+    loadSettings();
 
     clearIcon();
     updatePlaceHolderText();
+
+    // Hide icons by default
+    hideGoButton();
+    m_rssIcon->hide();
+    m_autofillIcon->hide();
 }
 
 void LocationBar::setWebView(TabbedWebView* view)
 {
     m_webView = view;
 
+    m_bookmarkIcon->setWebView(m_webView);
+    m_rssIcon->setWebView(m_webView);
+    m_siteIcon->setWebView(m_webView);
+    m_autofillIcon->setWebView(m_webView);
+
+    connect(m_webView, SIGNAL(loadStarted()), SLOT(onLoadStarted()));
     connect(m_webView, SIGNAL(loadProgress(int)), SLOT(onLoadProgress(int)));
     connect(m_webView, SIGNAL(loadFinished(bool)), SLOT(onLoadFinished()));
 }
@@ -103,27 +122,39 @@ void LocationBar::setWebView(TabbedWebView* view)
 void LocationBar::setText(const QString &text)
 {
     LineEdit::setText(text);
+    m_forcePaintEvent = true;
     setCursorPosition(0);
 }
 
 void LocationBar::updatePlaceHolderText()
 {
-    setPlaceholderText(tr("Enter URL address or search on %1").arg(mApp->searchEnginesManager()->activeEngine().name));
+    QString engineName = qzSettings->searchWithDefaultEngine ?
+                         mApp->searchEnginesManager()->defaultEngine().name :
+                         mApp->searchEnginesManager()->activeEngine().name;
+    setPlaceholderText(tr("Enter URL address or search on %1").arg(engineName));
 }
 
 void LocationBar::showCompletion(const QString &newText)
 {
+    m_inlineCompletionVisible = false;
+
     LineEdit::setText(newText);
 
     // Move cursor to the end
     end(false);
 }
 
+void LocationBar::completionPopupClosed()
+{
+    m_inlineCompletionVisible = false;
+    m_popupClosed = true;
+}
+
 QUrl LocationBar::createUrl()
 {
     QUrl urlToLoad;
 
-    //Check for Search Engine shortcut
+    // Check for Search Engine shortcut
     int firstSpacePos = text().indexOf(QLatin1Char(' '));
     if (firstSpacePos != -1) {
         QString shortcut = text().left(firstSpacePos);
@@ -135,6 +166,10 @@ QUrl LocationBar::createUrl()
         }
     }
 
+    if (isInlineCompletionVisible()) {
+        urlToLoad = WebView::guessUrlFromString(text() + m_completer.domainCompletion());
+    }
+
     if (urlToLoad.isEmpty()) {
         QUrl guessedUrl = WebView::guessUrlFromString(text());
         if (!guessedUrl.isEmpty()) {
@@ -150,27 +185,40 @@ QUrl LocationBar::createUrl()
 
 QString LocationBar::convertUrlToText(const QUrl &url) const
 {
-    QString stringUrl = qz_urlEncodeQueryString(url);
+    QString stringUrl = QzTools::urlEncodeQueryString(url);
 
     if (stringUrl == QLatin1String("qupzilla:speeddial") || stringUrl == QLatin1String("about:blank")) {
-        stringUrl = "";
+        stringUrl.clear();
     }
 
     return stringUrl;
 }
 
+bool LocationBar::isInlineCompletionVisible() const
+{
+    return m_inlineCompletionVisible && !m_completer.domainCompletion().isEmpty();
+}
+
 void LocationBar::urlEnter()
 {
+    const QUrl url = createUrl();
+    const QString urlString = convertUrlToText(url);
+
     m_completer.closePopup();
     m_webView->setFocus();
 
-    emit loadUrl(createUrl());
+    if (urlString != text()) {
+        setText(convertUrlToText(url));
+    }
+
+    emit loadUrl(url);
 }
 
 void LocationBar::textEdit()
 {
     if (!text().isEmpty()) {
         m_completer.complete(text());
+        m_inlineCompletionVisible = true;
     }
     else {
         m_completer.closePopup();
@@ -181,10 +229,6 @@ void LocationBar::textEdit()
 
 void LocationBar::showGoButton()
 {
-    if (m_goIcon->isVisible()) {
-        return;
-    }
-
     m_rssIconVisible = m_rssIcon->isVisible();
 
     m_bookmarkIcon->hide();
@@ -196,44 +240,14 @@ void LocationBar::showGoButton()
 
 void LocationBar::hideGoButton()
 {
-    if (!m_goIcon->isVisible()) {
-        return;
-    }
-
     m_rssIcon->setVisible(m_rssIconVisible);
     m_bookmarkIcon->show();
-    m_goIcon->hide();
 
-    updateTextMargins();
-}
-
-void LocationBar::showMostVisited()
-{
-    m_completer.complete(QString());
-}
-
-void LocationBar::showSiteInfo()
-{
-    QUrl url = p_QupZilla->weView()->url();
-
-    if (url.isEmpty() || url.scheme() == QLatin1String("qupzilla")) {
-        return;
+    if (!qzSettings->alwaysShowGoIcon) {
+        m_goIcon->hide();
     }
 
-    SiteInfoWidget* info = new SiteInfoWidget(p_QupZilla);
-    info->showAt(this);
-}
-
-void LocationBar::rssIconClicked()
-{
-    RSSWidget* rss = new RSSWidget(m_webView, this);
-    rss->showAt(this);
-}
-
-void LocationBar::bookmarkIconClicked()
-{
-    BookmarksWidget* bWidget = new BookmarksWidget(p_QupZilla, m_webView, this);
-    bWidget->showAt(this);
+    updateTextMargins();
 }
 
 void LocationBar::showRSSIcon(bool state)
@@ -249,7 +263,7 @@ void LocationBar::showUrl(const QUrl &url)
         return;
     }
 
-    const QString &stringUrl = convertUrlToText(url);
+    const QString stringUrl = convertUrlToText(url);
 
     if (stringUrl == text()) {
         return;
@@ -312,13 +326,11 @@ void LocationBar::contextMenuEvent(QContextMenuEvent* event)
     }
 
     QMenu* tempMenu = createStandardContextMenu();
-    m_menu->clear();
+    QMenu menu(this);
 
     int i = 0;
-    foreach(QAction * act, tempMenu->actions()) {
-        act->setParent(m_menu);
-        tempMenu->removeAction(act);
-        m_menu->addAction(act);
+    foreach (QAction* act, tempMenu->actions()) {
+        menu.addAction(act);
 
         switch (i) {
         case 0:
@@ -335,13 +347,13 @@ void LocationBar::contextMenuEvent(QContextMenuEvent* event)
             break;
         case 5:
             act->setIcon(QIcon::fromTheme("edit-paste"));
-            m_menu->addAction(act);
-            m_menu->addAction(m_pasteAndGoAction);
+            menu.addAction(act);
+            menu.addAction(m_pasteAndGoAction);
             break;
         case 6:
             act->setIcon(QIcon::fromTheme("edit-delete"));
-            m_menu->addAction(act);
-            m_menu->addAction(m_clearAction);
+            menu.addAction(act);
+            menu.addAction(m_clearAction);
             break;
         case 8:
             act->setIcon(QIcon::fromTheme("edit-select-all"));
@@ -350,23 +362,25 @@ void LocationBar::contextMenuEvent(QContextMenuEvent* event)
         ++i;
     }
 
-    tempMenu->deleteLater();
-
     m_pasteAndGoAction->setEnabled(!QApplication::clipboard()->text().isEmpty());
 
-    //Prevent choosing first option with double rightclick
+    // Prevent choosing first option with double rightclick
     QPoint pos = event->globalPos();
     QPoint p(pos.x(), pos.y() + 1);
-    m_menu->popup(p);
+    menu.exec(p);
+
+    tempMenu->deleteLater();
 }
 
 void LocationBar::focusInEvent(QFocusEvent* event)
 {
-    const QString &stringUrl = convertUrlToText(m_webView->url());
+    if (m_webView) {
+        const QString stringUrl = convertUrlToText(m_webView->url());
 
-    // Text has been edited, let's show go button
-    if (stringUrl != text()) {
-        showGoButton();
+        // Text has been edited, let's show go button
+        if (stringUrl != text()) {
+            showGoButton();
+        }
     }
 
     LineEdit::focusInEvent(event);
@@ -383,7 +397,7 @@ void LocationBar::dropEvent(QDropEvent* event)
             emit loadUrl(dropUrl);
 
             QFocusEvent event(QFocusEvent::FocusOut);
-            QLineEdit::focusOutEvent(&event);
+            LineEdit::focusOutEvent(&event);
             return;
         }
     }
@@ -396,23 +410,26 @@ void LocationBar::dropEvent(QDropEvent* event)
             emit loadUrl(dropUrl);
 
             QFocusEvent event(QFocusEvent::FocusOut);
-            QLineEdit::focusOutEvent(&event);
+            LineEdit::focusOutEvent(&event);
             return;
         }
 
     }
-    QLineEdit::dropEvent(event);
+
+    LineEdit::dropEvent(event);
 }
 
 void LocationBar::focusOutEvent(QFocusEvent* event)
 {
-    QLineEdit::focusOutEvent(event);
+    LineEdit::focusOutEvent(event);
 
     if (event->reason() == Qt::PopupFocusReason
             || (!selectedText().isEmpty() && event->reason() != Qt::TabFocusReason)) {
         return;
     }
 
+    m_popupClosed = false;
+    m_forcePaintEvent = true;
     setCursorPosition(0);
     hideGoButton();
 
@@ -421,26 +438,6 @@ void LocationBar::focusOutEvent(QFocusEvent* event)
     }
 }
 
-void LocationBar::mouseDoubleClickEvent(QMouseEvent* event)
-{
-    if (event->button() == Qt::LeftButton && qzSettings->selectAllOnDoubleClick) {
-        selectAll();
-    }
-    else {
-        QLineEdit::mouseDoubleClickEvent(event);
-    }
-}
-
-void LocationBar::mousePressEvent(QMouseEvent* event)
-{
-    if (cursorPosition() == 0 && qzSettings->selectAllOnClick) {
-        selectAll();
-        return;
-    }
-
-    LineEdit::mousePressEvent(event);
-}
-
 void LocationBar::keyPressEvent(QKeyEvent* event)
 {
     switch (event->key()) {
@@ -452,10 +449,48 @@ void LocationBar::keyPressEvent(QKeyEvent* event)
         }
         break;
 
+    case Qt::Key_A:
+        if (event->modifiers() == Qt::ControlModifier && m_inlineCompletionVisible) {
+            m_inlineCompletionVisible = false;
+        }
+        break;
+
     case Qt::Key_Down:
         m_completer.complete(text());
         break;
 
+    case Qt::Key_End:
+    case Qt::Key_Right: {
+        const QString completionText = m_completer.domainCompletion();
+        if (m_inlineCompletionVisible && !completionText.isEmpty()) {
+            m_inlineCompletionVisible = false;
+
+            setText(text() + completionText);
+            setCursorPosition(text().size());
+            m_completer.closePopup();
+        }
+
+        if (m_completer.isPopupVisible()) {
+            m_completer.closePopup();
+        }
+        break;
+    }
+
+    case Qt::Key_Left:
+        if (m_completer.isPopupVisible()) {
+            m_completer.closePopup();
+        }
+        break;
+
+    case Qt::Key_Delete:
+        if (m_inlineCompletionVisible) {
+            m_inlineCompletionVisible = false;
+
+            update();
+            event->accept();
+        }
+        break;
+
     case Qt::Key_Escape:
         m_webView->setFocus();
         showUrl(m_webView->url());
@@ -476,6 +511,7 @@ void LocationBar::keyPressEvent(QKeyEvent* event)
             break;
 
         case Qt::AltModifier:
+            m_completer.closePopup();
             p_QupZilla->tabWidget()->addView(createUrl());
             m_holdingAlt = false;
             break;
@@ -523,79 +559,239 @@ void LocationBar::keyReleaseEvent(QKeyEvent* event)
     LineEdit::keyReleaseEvent(event);
 }
 
-LocationBar::~LocationBar()
+void LocationBar::onLoadStarted()
 {
-    delete m_bookmarkIcon;
+    m_progressVisible = true;
+    m_autofillIcon->hide();
 }
 
 void LocationBar::onLoadProgress(int progress)
 {
     if (qzSettings->showLoadingProgress) {
-        m_loadFinished = false;
         m_loadProgress = progress;
-        repaint();
+        update();
     }
 }
 
 void LocationBar::onLoadFinished()
 {
     if (qzSettings->showLoadingProgress) {
-        m_loadFinished = false;
         QTimer::singleShot(700, this, SLOT(hideProgress()));
     }
+
+    WebPage* page = qobject_cast<WebPage*>(m_webView->page());
+
+    if (page && page->hasMultipleUsernames()) {
+        m_autofillIcon->setFormData(page->autoFillData());
+        m_autofillIcon->show();
+    }
+}
+
+void LocationBar::loadSettings()
+{
+    Settings settings;
+    settings.beginGroup("AddressBar");
+    m_progressStyle = static_cast<ProgressStyle>(settings.value("ProgressStyle", 0).toInt());
+    bool customColor = settings.value("UseCustomProgressColor", false).toBool();
+    m_progressColor = customColor ? settings.value("CustomProgressColor", palette().color(QPalette::Highlight)).value<QColor>() : QColor();
+    settings.endGroup();
+}
+
+void LocationBar::onMessage(Qz::AppMessageType msg, bool state)
+{
+    Q_UNUSED(state)
+    if (!qzSettings->showLoadingProgress) {
+        return;
+    }
+
+    if (msg == Qz::AM_ReloadSettings) {
+        loadSettings();
+    }
 }
 
 void LocationBar::hideProgress()
 {
-    if (qzSettings->showLoadingProgress) {
-        m_loadFinished = true;
-        repaint();
+    if (qzSettings->showLoadingProgress && m_loadProgress == 100) {
+        m_progressVisible = false;
+        update();
     }
 }
 
 void LocationBar::paintEvent(QPaintEvent* event)
 {
-    if (hasFocus() || !qzSettings->showLoadingProgress || m_loadFinished) {
-        LineEdit::paintEvent(event);
-        return;
-    }
-
     QStyleOptionFrameV3 option;
     initStyleOption(&option);
 
+    int lm, tm, rm, bm;
+    getTextMargins(&lm, &tm, &rm, &bm);
+
+    QRect contentsRect = style()->subElementRect(QStyle::SE_LineEditContents, &option, this);
+    contentsRect.adjust(lm, tm, -rm, -bm);
+
+    const QFontMetrics fm = fontMetrics();
+    const int x = contentsRect.x() + 4;
+    const int y = contentsRect.y() + (contentsRect.height() - fm.height() + 1) / 2;
+    const int width = contentsRect.width() - 6;
+    const int height = fm.height();
+    const QRect textRect(x, y, width, height);
+
+    QTextOption opt;
+    opt.setWrapMode(QTextOption::NoWrap);
+
     QPainter p(this);
     p.setRenderHint(QPainter::Antialiasing, true);
     p.setRenderHint(QPainter::TextAntialiasing, true);
 
+    if (hasFocus() && m_inlineCompletionVisible) {
+        // Draw inline domain completion if available
+        const QString completionText = m_completer.domainCompletion();
+
+        if (!completionText.isEmpty()) {
+            QRect completionRect = textRect;
+            completionRect.setX(completionRect.x() + fm.width(text()) + 1);
+            completionRect.setWidth(fm.width(completionText) + 1);
+
+            style()->drawPrimitive(QStyle::PE_PanelLineEdit, &option, &p, this);
+            // Text part
+            p.drawText(textRect, text(), opt);
+            // Completion part
+            p.fillRect(completionRect, palette().color(QPalette::Highlight));
+            p.setPen(palette().color(QPalette::HighlightedText));
+            p.drawText(completionRect, completionText, opt);
+            return;
+        }
+    }
+
+    if (m_completer.isPopupVisible() && !m_completer.showingMostVisited()) {
+        // We need to draw cursor when popup is visible
+        // But don't paint it if we are just showing most visited sites
+        const int cursorWidth = style()->pixelMetric(QStyle::PM_TextCursorWidth, &option, this);
+        const int cursorHeight = fm.height();
+
+        QString textPart = text().left(cursorPosition());
+        int cursorXpos = x + fontMetrics().width(textPart);
+
+        QRect cursor = cursorRect();
+        cursor.setX(cursorXpos + 1);
+        cursor.setWidth(cursorWidth);
+        cursor.setHeight(cursorHeight);
+
+        style()->drawPrimitive(QStyle::PE_PanelLineEdit, &option, &p, this);
+
+        QRect actualTextRect = textRect;
+        actualTextRect.setWidth(fontMetrics().width(text()) + 1);
+
+        // When popup is visible, Ctrl + A (Select All) is the only way to select text
+        if (selectedText() == text()) {
+            p.fillRect(actualTextRect, palette().color(QPalette::Highlight));
+            p.setPen(palette().color(QPalette::HighlightedText));
+        }
+
+        p.drawText(actualTextRect, text(), opt);
+
+        if (textRect.contains(cursor.center().x(), cursor.center().y())) {
+            p.fillRect(cursor, option.palette.text().color());
+        }
+        return;
+    }
+
+    if (hasFocus() || text().isEmpty() || m_forcePaintEvent) {
+        LineEdit::paintEvent(event);
+        if (m_forcePaintEvent) {
+            m_forcePaintEvent = false;
+            update();
+        }
+        return;
+    }
+
     style()->drawPrimitive(QStyle::PE_PanelLineEdit, &option, &p, this);
 
-    QRect contentsRect = style()->subElementRect(QStyle::SE_LineEditContents, &option, this);
-    int lm, tm, rm, bm;
-    getTextMargins(&lm, &tm, &rm, &bm);
-    contentsRect.adjust(lm, tm, -rm, -bm);
-    QFontMetrics fm = fontMetrics();
-    const int x = contentsRect.x() + 3;
-    const int y = contentsRect.y() + (contentsRect.height() - fm.height() + 1) / 2;
-    const int width = contentsRect.width() - 6;
-    const int height = fm.height();
-    QRect textRect(x, y, width, height);
-
-    QColor bg = palette().color(QPalette::Base);
-    if (!bg.isValid() || bg.alpha() == 0) {
-        bg = p_QupZilla->palette().color(QPalette::Base);
-    }
-    bg = bg.darker(110);
-    p.setBrush(QBrush(bg));
-
     QPen oldPen = p.pen();
-    QPen outlinePen(bg.darker(110), 0.8);
-    p.setPen(outlinePen);
 
-    QRect bar = textRect.adjusted(-3, 0, 6 - (textRect.width() * (100.0 - m_loadProgress) / 100), 0);
-    const int roundness = bar.height() / 4.0;
-    p.drawRoundedRect(bar, roundness, roundness);
+    if (qzSettings->showLoadingProgress && m_progressVisible) {
+        QColor bg = m_progressColor;
+        if (!bg.isValid() || bg.alpha() == 0) {
+            bg = Colors::mid(palette().color(QPalette::Base),
+                             palette().color(QPalette::Text),
+                             m_progressStyle > 0 ? 4 : 8, 1);
+        }
+        p.setBrush(QBrush(bg));
+
+        QPen outlinePen(bg.darker(110), 0.8);
+        p.setPen(outlinePen);
+
+        switch (m_progressStyle) {
+        case ProgressFilled: {
+            QRect bar = contentsRect.adjusted(0, 1, 0, -1);
+            bar.setWidth(bar.width() * m_loadProgress / 100);
+            const int roundness = bar.height() / 4.0;
+            p.drawRoundedRect(bar, roundness, roundness);
+            break;
+        }
+        case ProgressBottom: {
+            outlinePen.setWidthF(0.3);
+            outlinePen.setColor(outlinePen.color().darker(130));
+            p.setPen(outlinePen);
+            QRect bar(contentsRect.x(), contentsRect.bottom() - 3,
+                      contentsRect.width() * m_loadProgress / 100.0, 3);
+            p.drawRoundedRect(bar, 1, 1);
+            break;
+        }
+        case ProgressTop: {
+            outlinePen.setWidthF(0.3);
+            outlinePen.setColor(outlinePen.color().darker(130));
+            p.setPen(outlinePen);
+            QRect bar(contentsRect.x(), contentsRect.top() + 1,
+                      contentsRect.width() * m_loadProgress / 100.0, 3);
+            p.drawRoundedRect(bar, 1, 1);
+            break;
+        }
+        default:
+            break;
+        }
+    }
 
     p.setPen(oldPen);
-//    Qt::Alignment va = QStyle::visualAlignment(QApplication::layoutDirection(), QFlag(alignment()));
-    p.drawText(textRect, text());
+
+    const QString hostName = m_webView->url().host();
+    QString currentText = text();
+    QRect currentRect = textRect;
+    if (!hostName.isEmpty()) {
+        const int hostPos = currentText.indexOf(hostName);
+        if (hostPos > 0) {
+            QPen lightPen = oldPen;
+            QColor lightColor = Colors::mid(palette().color(QPalette::Base),
+                                            palette().color(QPalette::Text),
+                                            1, 1);
+            lightPen.setColor(lightColor);
+
+            p.setPen(lightPen);
+            currentText = text().mid(0, hostPos);
+            currentRect.setWidth(fm.width(currentText));
+            p.drawText(currentRect, currentText, opt);
+
+            p.setPen(oldPen);
+            currentRect.setX(currentRect.x() + currentRect.width());
+            const int hostWidth = fm.width(hostName);
+            currentRect.setWidth(hostWidth);
+            p.drawText(currentRect, hostName, opt);
+
+            p.setFont(font());
+            currentText = text().mid(hostPos + hostName.length());
+            currentRect.setX(currentRect.x() + hostWidth);
+            currentRect.setWidth(textRect.width() - currentRect.x() + textRect.x());
+            p.setPen(lightPen);
+
+            if (currentText.isRightToLeft()) {
+                // Insert unicode control characters: prepend LRE then append LRM+PDF
+                currentText.prepend(QChar(0x202A)).append(QChar(0x200E)).append(QChar(0x202C));
+            }
+        }
+    }
+
+    p.drawText(currentRect, currentText, opt);
+}
+
+LocationBar::~LocationBar()
+{
 }
