@@ -1,6 +1,6 @@
 /* ============================================================
 * QupZilla - WebKit based browser
-* Copyright (C) 2010-2012  David Rosca <nowrep@gmail.com>
+* Copyright (C) 2010-2014  David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ============================================================ */
 #include "qupzillaschemehandler.h"
-#include "globalfunctions.h"
+#include "qztools.h"
 #include "qupzilla.h"
 #include "mainapplication.h"
 #include "tabbedwebview.h"
@@ -27,12 +27,12 @@
 #include "settings.h"
 #include "iconprovider.h"
 
-#include <QTextDocument>
 #include <QTextStream>
 #include <QTimer>
 #include <QSettings>
+#include <QWebSecurityOrigin>
 
-QString authorString(const char* name, const QString &mail)
+static QString authorString(const char* name, const QString &mail)
 {
     return QString("%1 &lt;<a href=\"mailto:%2\">%2</a>&gt;").arg(QString::fromUtf8(name), mail);
 }
@@ -79,6 +79,7 @@ QupZillaSchemeReply::QupZillaSchemeReply(const QNetworkRequest &req, QObject* pa
 
 void QupZillaSchemeReply::loadPage()
 {
+    QWebSecurityOrigin::addLocalScheme("qupzilla");
     QTextStream stream(&m_buffer);
     stream.setCodec("UTF-8");
 
@@ -113,6 +114,7 @@ void QupZillaSchemeReply::loadPage()
 
     emit readyRead();
     emit finished();
+    QWebSecurityOrigin::removeLocalScheme("qupzilla");
 }
 
 void QupZillaSchemeReply::delayedFinish()
@@ -139,7 +141,7 @@ QString QupZillaSchemeReply::reportbugPage()
         return bPage;
     }
 
-    bPage.append(qz_readAllFileContents(":html/reportbug.html"));
+    bPage.append(QzTools::readAllFileContents(":html/reportbug.html"));
     bPage.replace(QLatin1String("%FAVICON%"), QLatin1String("qrc:icons/qupzilla.png"));
     bPage.replace(QLatin1String("%BOX-BORDER%"), QLatin1String("qrc:html/box-border.png"));
 
@@ -151,9 +153,19 @@ QString QupZillaSchemeReply::reportbugPage()
     bPage.replace(QLatin1String("%TYPE%"), tr("Issue type"));
     bPage.replace(QLatin1String("%DESCRIPTION%"), tr("Issue description"));
     bPage.replace(QLatin1String("%SEND%"), tr("Send"));
-    bPage.replace(QLatin1String("%E-MAIL-OPTIONAL%"), tr("E-mail is optional<br/><b>Note: </b>Please read how to make a bug report <a href=%1>here</a> first.").arg("https://github.com/QupZilla/qupzilla/wiki/Bug-Reports target=_blank"));
+    bPage.replace(QLatin1String("%E-MAIL-OPTIONAL%"), tr("E-mail is optional<br/><b>Note: </b>Please read how to make a "
+                  "bug report <a href=%1>here</a> first.").arg("https://github.com/QupZilla/qupzilla/wiki/Bug-Reports target=_blank"));
     bPage.replace(QLatin1String("%FIELDS-ARE-REQUIRED%"), tr("Please fill out all required fields!"));
-    bPage = qz_applyDirectionToPage(bPage);
+
+    bPage.replace(QLatin1String("%INFO_OS%"), QzTools::operatingSystem());
+    bPage.replace(QLatin1String("%INFO_APP%"), QupZilla::VERSION
+#ifdef GIT_REVISION
+                  + " (" + GIT_REVISION + ")"
+#endif
+                 );
+    bPage.replace(QLatin1String("%INFO_QT%"), QString("%1 (built with %2)").arg(qVersion(), QT_VERSION_STR));
+    bPage.replace(QLatin1String("%INFO_WEBKIT%"), QupZilla::WEBKITVERSION),
+                  bPage = QzTools::applyDirectionToPage(bPage);
 
     return bPage;
 }
@@ -166,7 +178,7 @@ QString QupZillaSchemeReply::startPage()
         return sPage;
     }
 
-    sPage.append(qz_readAllFileContents(":html/start.html"));
+    sPage.append(QzTools::readAllFileContents(":html/start.html"));
     sPage.replace(QLatin1String("%FAVICON%"), QLatin1String("qrc:icons/qupzilla.png"));
     sPage.replace(QLatin1String("%BOX-BORDER%"), QLatin1String("qrc:html/box-border.png"));
     sPage.replace(QLatin1String("%ABOUT-IMG%"), QLatin1String("qrc:icons/other/about.png"));
@@ -177,7 +189,7 @@ QString QupZillaSchemeReply::startPage()
     sPage.replace(QLatin1String("%WWW%"), QupZilla::WIKIADDRESS);
     sPage.replace(QLatin1String("%ABOUT-QUPZILLA%"), tr("About QupZilla"));
     sPage.replace(QLatin1String("%PRIVATE-BROWSING%"), mApp->isPrivateSession() ? tr("<h1>Private Browsing</h1>") : QString());
-    sPage = qz_applyDirectionToPage(sPage);
+    sPage = QzTools::applyDirectionToPage(sPage);
 
     return sPage;
 }
@@ -187,11 +199,11 @@ QString QupZillaSchemeReply::aboutPage()
     static QString aPage;
 
     if (aPage.isEmpty()) {
-        aPage.append(qz_readAllFileContents(":html/about.html"));
+        aPage.append(QzTools::readAllFileContents(":html/about.html"));
         aPage.replace(QLatin1String("%FAVICON%"), QLatin1String("qrc:icons/qupzilla.png"));
         aPage.replace(QLatin1String("%BOX-BORDER%"), QLatin1String("qrc:html/box-border.png"));
         aPage.replace(QLatin1String("%ABOUT-IMG%"), QLatin1String("qrc:icons/other/about.png"));
-        aPage.replace(QLatin1String("%COPYRIGHT-INCLUDE%"), Qt::escape(qz_readAllFileContents(":html/copyright")));
+        aPage.replace(QLatin1String("%COPYRIGHT-INCLUDE%"), QzTools::escape(QzTools::readAllFileContents(":html/copyright")));
 
         aPage.replace(QLatin1String("%TITLE%"), tr("About QupZilla"));
         aPage.replace(QLatin1String("%ABOUT-QUPZILLA%"), tr("About QupZilla"));
@@ -209,7 +221,7 @@ QString QupZillaSchemeReply::aboutPage()
         aPage.replace(QLatin1String("%MAIN-DEVELOPER-TEXT%"), authorString(QupZilla::AUTHOR.toUtf8(), "nowrep@gmail.com"));
         aPage.replace(QLatin1String("%CONTRIBUTORS%"), tr("Contributors"));
         aPage.replace(QLatin1String("%CONTRIBUTORS-TEXT%"),
-                      authorString("Mladen Pejaković", "pejakm@gmail.com") + "<br/>" +
+                      authorString("Mladen Pejaković", "pejakm@autistici.org") + "<br/>" +
                       authorString("Alexander Samilov", "alexsamilovskih@gmail.com") + "<br/>" +
                       authorString("Seyyed Razi Alavizadeh", "s.r.alavizadeh@gmail.com") + "<br/>" +
                       authorString("Franz Fellner", "alpine.art.de@googlemail.com") + "<br/>" +
@@ -237,8 +249,9 @@ QString QupZillaSchemeReply::aboutPage()
                       authorString("Oleg Brezhnev", "oleg-423@yandex.ru") + " (Russian)<br/>" +
                       authorString("Sérgio Marques", "smarquespt@gmail.com") + " (Portuguese)<br/>" +
                       authorString("Alexandre Carvalho", "alexandre05@live.com") + " (Brazilian Portuguese)<br/>" +
-                      authorString("Mladen Pejaković", "pejakm@gmail.com") + " (Serbian)<br/>" +
+                      authorString("Mladen Pejaković", "pejakm@autistici.org") + " (Serbian)<br/>" +
                       authorString("Unink-Lio", "unink4451@163.com") + " (Chinese)<br/>" +
+                      authorString("Yu Hai", "yohanprc@eml.cc") + " (Chinese)<br/>" +
                       authorString("Wu Cheng-Hong", "stu2731652@gmail.com") + " (Traditional Chinese)<br/>" +
                       authorString("Widya Walesa", "walecha99@gmail.com") + " (Indonesian)<br/>" +
                       authorString("Beqa Arabuli", "arabulibeqa@gmail.com") + " (Georgian)<br/>" +
@@ -246,9 +259,15 @@ QString QupZillaSchemeReply::aboutPage()
                       authorString("Gábor Oberle", "oberleg@myopera.com") + " (Hungarian)<br/>" +
                       authorString("Piccoro McKay Lenz", "mckaygerhard@gmail.com") + " (Venezulean Spanish)<br/>" +
                       authorString("Stanislav Kuznietsov", "stanislav_kuznetsov@ukr.net") + " (Ukrainian)<br/>" +
-                      authorString("Seyyed Razi Alavizadeh", "s.r.alavizadeh@gmail.com") + " (Persian)"
+                      authorString("Seyyed Razi Alavizadeh", "s.r.alavizadeh@gmail.com") + " (Persian)<br/>" +
+                      authorString("Guillem Prats", "guprej@gmail.com") + " (Catalan)<br/>" +
+                      authorString("Clara Villalba", "cvilmon@gmail.com") + " (Catalan)<br/>" +
+                      authorString("Muhammad Fawwaz Orabi", "mfawwaz93@gmail.com") + " (Arabic)<br/>" +
+                      authorString("Lasso Kante", "kantemou@gmail.com") + " (N'ko)<br/>" +
+                      authorString("Kizito Birabwa", "kbirabwa@yahoo.co.uk") + " (Luganda)<br/>" +
+                      authorString("Xabier Aramendi", "azpidatziak@gmail.com") + " (Basque)"
                      );
-        aPage = qz_applyDirectionToPage(aPage);
+        aPage = QzTools::applyDirectionToPage(aPage);
     }
 
     return aPage;
@@ -259,7 +278,7 @@ QString QupZillaSchemeReply::speeddialPage()
     static QString dPage;
 
     if (dPage.isEmpty()) {
-        dPage.append(qz_readAllFileContents(":html/speeddial.html"));
+        dPage.append(QzTools::readAllFileContents(":html/speeddial.html"));
         dPage.replace(QLatin1String("%FAVICON%"), QLatin1String("qrc:icons/qupzilla.png"));
         dPage.replace(QLatin1String("%IMG_PLUS%"), QLatin1String("qrc:html/plus.png"));
         dPage.replace(QLatin1String("%BOX-BORDER%"), QLatin1String("qrc:html/box-border-small.png"));
@@ -269,7 +288,7 @@ QString QupZillaSchemeReply::speeddialPage()
         dPage.replace(QLatin1String("%JQUERY%"), QLatin1String("qrc:html/jquery.js"));
         dPage.replace(QLatin1String("%JQUERY-UI%"), QLatin1String("qrc:html/jquery-ui.js"));
         dPage.replace(QLatin1String("%LOADING-IMG%"), QLatin1String("qrc:html/loading.gif"));
-        dPage.replace(QLatin1String("%IMG_SETTINGS%"), QLatin1String("qrc:html/setting.png"));
+        dPage.replace(QLatin1String("%IMG_SETTINGS%"), QLatin1String("qrc:html/configure.png"));
 
         dPage.replace(QLatin1String("%SITE-TITLE%"), tr("Speed Dial"));
         dPage.replace(QLatin1String("%ADD-TITLE%"), tr("Add New Page"));
@@ -295,7 +314,7 @@ QString QupZillaSchemeReply::speeddialPage()
         dPage.replace(QLatin1String("%TXT_NRROWS%"), tr("Maximum pages in a row:"));
         dPage.replace(QLatin1String("%TXT_SDSIZE%"), tr("Change size of pages:"));
         dPage.replace(QLatin1String("%TXT_CNTRDLS%"), tr("Center speed dials"));
-        dPage = qz_applyDirectionToPage(dPage);
+        dPage = QzTools::applyDirectionToPage(dPage);
     }
 
     QString page = dPage;
@@ -316,10 +335,10 @@ QString QupZillaSchemeReply::restorePage()
     static QString rPage;
 
     if (rPage.isEmpty()) {
-        rPage.append(qz_readAllFileContents(":html/restore.html"));
+        rPage.append(QzTools::readAllFileContents(":html/restore.html"));
         rPage.replace(QLatin1String("%FAVICON%"), QLatin1String("qrc:icons/qupzilla.png"));
         rPage.replace(QLatin1String("%BOX-BORDER%"), QLatin1String("qrc:html/box-border.png"));
-        rPage.replace(QLatin1String("%IMAGE%"), qz_pixmapToByteArray(qIconProvider->standardIcon(QStyle::SP_MessageBoxWarning).pixmap(45, 45)));
+        rPage.replace(QLatin1String("%IMAGE%"), QzTools::pixmapToByteArray(qIconProvider->standardIcon(QStyle::SP_MessageBoxWarning).pixmap(45, 45)));
 
 
         rPage.replace(QLatin1String("%TITLE%"), tr("Restore Session"));
@@ -327,7 +346,7 @@ QString QupZillaSchemeReply::restorePage()
         rPage.replace(QLatin1String("%APOLOGIZE%"), tr("We apologize for this. Would you like to restore the last saved state?"));
         rPage.replace(QLatin1String("%TRY-REMOVING%"), tr("Try removing one or more tabs that you think cause troubles"));
         rPage.replace(QLatin1String("%START-NEW%"), tr("Or you can start completely new session"));
-        rPage = qz_applyDirectionToPage(rPage);
+        rPage = QzTools::applyDirectionToPage(rPage);
     }
 
     return rPage;
@@ -338,7 +357,7 @@ QString QupZillaSchemeReply::configPage()
     static QString cPage;
 
     if (cPage.isEmpty()) {
-        cPage.append(qz_readAllFileContents(":html/config.html"));
+        cPage.append(QzTools::readAllFileContents(":html/config.html"));
         cPage.replace(QLatin1String("%FAVICON%"), QLatin1String("qrc:icons/qupzilla.png"));
         cPage.replace(QLatin1String("%BOX-BORDER%"), QLatin1String("qrc:html/box-border.png"));
         cPage.replace(QLatin1String("%ABOUT-IMG%"), QLatin1String("qrc:icons/other/about.png"));
@@ -368,7 +387,7 @@ QString QupZillaSchemeReply::configPage()
                       QString("<dt>%1</dt><dd>%2<dd>").arg(tr("Qt version"), QT_VERSION_STR) +
                       QString("<dt>%1</dt><dd>%2<dd>").arg(tr("WebKit version"), QupZilla::WEBKITVERSION) +
                       QString("<dt>%1</dt><dd>%2<dd>").arg(tr("Build time"), QupZilla::BUILDTIME) +
-                      QString("<dt>%1</dt><dd>%2<dd>").arg(tr("Platform"), qz_buildSystem()));
+                      QString("<dt>%1</dt><dd>%2<dd>").arg(tr("Platform"), QzTools::operatingSystem()));
 
         cPage.replace(QLatin1String("%PATHS-TEXT%"),
                       QString("<dt>%1</dt><dd>%2<dd>").arg(tr("Profile"), mApp->currentProfilePath()) +
@@ -379,28 +398,32 @@ QString QupZillaSchemeReply::configPage()
                       QString("<dt>%1</dt><dd>%2<dd>").arg(tr("Themes"), mApp->THEMESDIR) +
                       QString("<dt>%1</dt><dd>%2<dd>").arg(tr("Translations"), mApp->TRANSLATIONSDIR));
 
-        QString debugBuild = tr("Disabled");
-        QString webGLEnabled = tr("Disabled");
-        QString w7APIEnabled = tr("Disabled");
-        QString KDEIntegration = tr("Disabled");
-        QString portableBuild = tr("Disabled");
-
 #ifdef QUPZILLA_DEBUG_BUILD
-        debugBuild = tr("<b>Enabled</b>");
+        QString debugBuild = tr("<b>Enabled</b>");
+#else
+        QString debugBuild = tr("Disabled");
 #endif
-#ifdef USE_WEBGL
-        webGLEnabled = tr("<b>Enabled</b>");
+
+#if defined (USE_WEBGL) || (QTWEBKIT_FROM_2_3 && defined(QZ_WS_X11))
+        QString webGLEnabled = tr("<b>Enabled</b>");
+#else
+        QString webGLEnabled = tr("Disabled");
 #endif
+
 #if defined(Q_OS_WIN) && defined(W7API)
-        w7APIEnabled = tr("<b>Enabled</b>");
+        QString w7APIEnabled = tr("<b>Enabled</b>");
+#else
+        QString w7APIEnabled = tr("Disabled");
 #endif
-#if defined(Q_WS_X11) && defined(KDE)
-        KDEIntegration = tr("<b>Enabled</b>");
-#endif
-#ifdef PORTABLE_BUILD
-        portableBuild = tr("<b>Enabled</b>");
+
+#if defined(QZ_WS_X11) && defined(KDE_INTEGRATION)
+        QString KDEIntegration = tr("<b>Enabled</b>");
+#else
+        QString KDEIntegration = tr("Disabled");
 #endif
 
+        QString portableBuild = mApp->isPortable() ? tr("<b>Enabled</b>") : tr("Disabled");
+
         cPage.replace(QLatin1String("%BUILD-CONFIG-TEXT%"),
                       QString("<dt>%1</dt><dd>%2<dd>").arg(tr("Debug build"), debugBuild) +
                       QString("<dt>%1</dt><dd>%2<dd>").arg(tr("WebGL support"), webGLEnabled) +
@@ -408,7 +431,7 @@ QString QupZillaSchemeReply::configPage()
                       QString("<dt>%1</dt><dd>%2<dd>").arg(tr("KDE integration"), KDEIntegration) +
                       QString("<dt>%1</dt><dd>%2<dd>").arg(tr("Portable build"), portableBuild));
 
-        cPage = qz_applyDirectionToPage(cPage);
+        cPage = QzTools::applyDirectionToPage(cPage);
     }
 
     QString page = cPage;
@@ -417,10 +440,10 @@ QString QupZillaSchemeReply::configPage()
     QString pluginsString;
     const QList<Plugins::Plugin> &availablePlugins = mApp->plugins()->getAvailablePlugins();
 
-    foreach(const Plugins::Plugin & plugin, availablePlugins) {
+    foreach (const Plugins::Plugin &plugin, availablePlugins) {
         PluginSpec spec = plugin.pluginSpec;
         pluginsString.append(QString("<tr><td>%1</td><td>%2</td><td>%3</td><td>%4</td></tr>").arg(
-                                 spec.name, spec.version, Qt::escape(spec.author), spec.description));
+                                 spec.name, spec.version, QzTools::escape(spec.author), spec.description));
     }
 
     if (pluginsString.isEmpty()) {
@@ -431,12 +454,12 @@ QString QupZillaSchemeReply::configPage()
 
     QString allGroupsString;
     QSettings* settings = Settings::globalSettings();
-    foreach(const QString & group, settings->childGroups()) {
+    foreach (const QString &group, settings->childGroups()) {
         QString groupString = QString("<tr><th colspan=\"2\">[%1]</th></tr>").arg(group);
         settings->beginGroup(group);
 
-        foreach(const QString & key, settings->childKeys()) {
-            const QVariant &keyValue = settings->value(key);
+        foreach (const QString &key, settings->childKeys()) {
+            const QVariant keyValue = settings->value(key);
             QString keyString;
 
             switch (keyValue.type()) {
@@ -462,7 +485,7 @@ QString QupZillaSchemeReply::configPage()
                 keyString = QLatin1String("\"empty\"");
             }
 
-            groupString.append(QString("<tr><td>%1</td><td>%2</td></tr>").arg(key, Qt::escape(keyString)));
+            groupString.append(QString("<tr><td>%1</td><td>%2</td></tr>").arg(key, QzTools::escape(keyString)));
         }
 
         settings->endGroup();
