@@ -1,6 +1,6 @@
 /* ============================================================
 * QupZilla - WebKit based browser
-* Copyright (C) 2010-2012  David Rosca <nowrep@gmail.com>
+* Copyright (C) 2010-2014  David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -30,6 +30,7 @@
 #include "pluginsmanager.h"
 #include "qtwin.h"
 #include "sslmanager.h"
+#include "jsoptions.h"
 #include "networkproxyfactory.h"
 #include "networkmanager.h"
 #include "desktopnotificationsfactory.h"
@@ -37,27 +38,53 @@
 #include "navigationbar.h"
 #include "thememanager.h"
 #include "acceptlanguage.h"
-#include "globalfunctions.h"
-#include "autofillmodel.h"
+#include "qztools.h"
+#include "autofill.h"
 #include "settings.h"
 #include "tabbedwebview.h"
 #include "clearprivatedata.h"
 #include "useragentdialog.h"
+#include "registerqappassociation.h"
+#include "html5permissions/html5permissionsdialog.h"
+#include "pac/pacmanager.h"
 
 #include <QSettings>
 #include <QInputDialog>
 #include <QFileDialog>
 #include <QMessageBox>
 #include <QCloseEvent>
+#include <QColorDialog>
+#include <QDesktopWidget>
+
+static QString createLanguageItem(const QString &lang)
+{
+    QLocale locale(lang);
+
+    QString country = QLocale::countryToString(locale.country());
+    QString language = QLocale::languageToString(locale.language());
+
+    if (lang == QLatin1String("nqo")) {
+        return QString("N'ko (nqo)");
+    }
+
+    return QString("%1, %2 (%3)").arg(language, country, lang);
+}
 
 Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     : QDialog(parent)
     , ui(new Ui::Preferences)
     , p_QupZilla(mainClass)
+    , m_autoFillManager(0)
     , m_pluginsList(0)
+    , m_autoFillEnabled(false)
 {
     setAttribute(Qt::WA_DeleteOnClose);
     ui->setupUi(this);
+    ui->languages->setLayoutDirection(Qt::LeftToRight);
+
+    m_themesManager = new ThemeManager(ui->themesWidget, this);
+    m_pluginsList = new PluginsManager(this);
+    ui->pluginsFrame->addWidget(m_pluginsList);
 
     if (QIcon::themeName().toLower() == QLatin1String("oxygen")) {
         ui->listWidget->item(0)->setIcon(QIcon::fromTheme("preferences-desktop", QIcon(":/icons/preferences/preferences-desktop.png")));
@@ -65,12 +92,13 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
         ui->listWidget->item(2)->setIcon(QIcon::fromTheme("tab-new-background", QIcon(":/icons/preferences/applications-internet.png")));
         ui->listWidget->item(3)->setIcon(QIcon::fromTheme("preferences-system-network", QIcon(":/icons/preferences/applications-webbrowsers.png")));
         ui->listWidget->item(4)->setIcon(QIcon::fromTheme("preferences-desktop-font", QIcon(":/icons/preferences/applications-fonts.png")));
-        ui->listWidget->item(5)->setIcon(QIcon::fromTheme("download", QIcon(":/icons/preferences/mail-inbox.png")));
-        ui->listWidget->item(6)->setIcon(QIcon::fromTheme("user-identity", QIcon(":/icons/preferences/dialog-password.png")));
-        ui->listWidget->item(7)->setIcon(QIcon::fromTheme("preferences-system-firewall", QIcon(":/icons/preferences/preferences-system-firewall.png")));
-        ui->listWidget->item(8)->setIcon(QIcon::fromTheme("preferences-desktop-notification", QIcon(":/icons/preferences/dialog-question.png")));
-        ui->listWidget->item(9)->setIcon(QIcon::fromTheme("preferences-plugin", QIcon(":/icons/preferences/extension.png")));
-        ui->listWidget->item(10)->setIcon(QIcon::fromTheme("applications-system", QIcon(":/icons/preferences/applications-system.png")));
+        ui->listWidget->item(5)->setIcon(QIcon::fromTheme("configure-shortcuts", QIcon(":/icons/preferences/preferences-desktop-keyboard-shortcuts.png")));
+        ui->listWidget->item(6)->setIcon(QIcon::fromTheme("download", QIcon(":/icons/preferences/mail-inbox.png")));
+        ui->listWidget->item(7)->setIcon(QIcon::fromTheme("user-identity", QIcon(":/icons/preferences/dialog-password.png")));
+        ui->listWidget->item(8)->setIcon(QIcon::fromTheme("preferences-system-firewall", QIcon(":/icons/preferences/preferences-system-firewall.png")));
+        ui->listWidget->item(9)->setIcon(QIcon::fromTheme("preferences-desktop-notification", QIcon(":/icons/preferences/dialog-question.png")));
+        ui->listWidget->item(10)->setIcon(QIcon::fromTheme("preferences-plugin", QIcon(":/icons/preferences/extension.png")));
+        ui->listWidget->item(11)->setIcon(QIcon::fromTheme("applications-system", QIcon(":/icons/preferences/applications-system.png")));
     }
     else {
         ui->listWidget->item(0)->setIcon(QIcon::fromTheme("preferences-desktop", QIcon(":/icons/preferences/preferences-desktop.png")));
@@ -78,12 +106,13 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
         ui->listWidget->item(2)->setIcon(QIcon::fromTheme("applications-internet", QIcon(":/icons/preferences/applications-internet.png")));
         ui->listWidget->item(3)->setIcon(QIcon::fromTheme("applications-webbrowsers", QIcon(":/icons/preferences/applications-webbrowsers.png")));
         ui->listWidget->item(4)->setIcon(QIcon::fromTheme("applications-fonts", QIcon(":/icons/preferences/applications-fonts.png")));
-        ui->listWidget->item(5)->setIcon(QIcon::fromTheme("mail-inbox", QIcon(":/icons/preferences/mail-inbox.png")));
-        ui->listWidget->item(6)->setIcon(QIcon::fromTheme("dialog-password", QIcon(":/icons/preferences/dialog-password.png")));
-        ui->listWidget->item(7)->setIcon(QIcon::fromTheme("preferences-system-firewall", QIcon(":/icons/preferences/preferences-system-firewall.png")));
-        ui->listWidget->item(8)->setIcon(QIcon::fromTheme("dialog-question", QIcon(":/icons/preferences/dialog-question.png")));
-        ui->listWidget->item(9)->setIcon(QIcon::fromTheme("extension", QIcon(":/icons/preferences/extension.png")));
-        ui->listWidget->item(10)->setIcon(QIcon::fromTheme("applications-system", QIcon(":/icons/preferences/applications-system.png")));
+        ui->listWidget->item(5)->setIcon(QIcon::fromTheme("preferences-desktop-keyboard-shortcuts", QIcon(":/icons/preferences/preferences-desktop-keyboard-shortcuts.png")));
+        ui->listWidget->item(6)->setIcon(QIcon::fromTheme("mail-inbox", QIcon(":/icons/preferences/mail-inbox.png")));
+        ui->listWidget->item(7)->setIcon(QIcon::fromTheme("dialog-password", QIcon(":/icons/preferences/dialog-password.png")));
+        ui->listWidget->item(8)->setIcon(QIcon::fromTheme("preferences-system-firewall", QIcon(":/icons/preferences/preferences-system-firewall.png")));
+        ui->listWidget->item(9)->setIcon(QIcon::fromTheme("dialog-question", QIcon(":/icons/preferences/dialog-question.png")));
+        ui->listWidget->item(10)->setIcon(QIcon::fromTheme("extension", QIcon(":/icons/preferences/extension.png")));
+        ui->listWidget->item(11)->setIcon(QIcon::fromTheme("applications-system", QIcon(":/icons/preferences/applications-system.png")));
     }
 
     Settings settings;
@@ -99,6 +128,25 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     ui->checkUpdates->setChecked(settings.value("Web-Browser-Settings/CheckUpdates", DEFAULT_CHECK_UPDATES).toBool());
     ui->dontLoadTabsUntilSelected->setChecked(settings.value("Web-Browser-Settings/LoadTabsOnActivation", false).toBool());
 
+#if defined(Q_OS_WIN) && !defined(Q_OS_OS2)
+    ui->checkDefaultBrowser->setChecked(settings.value("Web-Browser-Settings/CheckDefaultBrowser", DEFAULT_CHECK_DEFAULTBROWSER).toBool());
+    if (mApp->associationManager()->isDefaultForAllCapabilities()) {
+        ui->checkNowDefaultBrowser->setText(tr("Default"));
+        ui->checkNowDefaultBrowser->setEnabled(false);
+    }
+    else {
+        ui->checkNowDefaultBrowser->setText(tr("Set as default"));
+        ui->checkNowDefaultBrowser->setEnabled(true);
+        connect(ui->checkNowDefaultBrowser, SIGNAL(clicked()), this, SLOT(makeQupZillaDefault()));
+    }
+#else // No Default Browser settings on non-Windows platform
+    ui->hSpacerDefaultBrowser->changeSize(0, 0, QSizePolicy::Fixed, QSizePolicy::Fixed);
+    ui->hLayoutDefaultBrowser->invalidate();
+    delete ui->hLayoutDefaultBrowser;
+    delete ui->checkDefaultBrowser;
+    delete ui->checkNowDefaultBrowser;
+#endif
+
     ui->newTabFrame->setVisible(false);
     if (m_newTabUrl.isEmpty()) {
         ui->newTab->setCurrentIndex(0);
@@ -117,9 +165,14 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     afterLaunchChanged(ui->afterLaunch->currentIndex());
     connect(ui->afterLaunch, SIGNAL(currentIndexChanged(int)), this, SLOT(afterLaunchChanged(int)));
     connect(ui->newTab, SIGNAL(currentIndexChanged(int)), this, SLOT(newTabChanged(int)));
-    connect(ui->useCurrentBut, SIGNAL(clicked()), this, SLOT(useActualHomepage()));
-    connect(ui->newTabUseCurrent, SIGNAL(clicked()), this, SLOT(useActualNewTab()));
-
+    if (p_QupZilla) {
+        connect(ui->useCurrentBut, SIGNAL(clicked()), this, SLOT(useActualHomepage()));
+        connect(ui->newTabUseCurrent, SIGNAL(clicked()), this, SLOT(useActualNewTab()));
+    }
+    else {
+        ui->useCurrentBut->setEnabled(false);
+        ui->newTabUseCurrent->setEnabled(false);
+    }
     //PROFILES
     m_actProfileName = mApp->currentProfilePath();
     m_actProfileName = m_actProfileName.left(m_actProfileName.length() - 1);
@@ -134,7 +187,7 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     ui->startProfile->addItem(actProfileName);
     QDir profilesDir(mApp->PROFILEDIR + "profiles/");
     QStringList list_ = profilesDir.entryList(QDir::Dirs | QDir::NoDotAndDotDot);
-    foreach(const QString & name, list_) {
+    foreach (const QString &name, list_) {
         if (actProfileName == name) {
             continue;
         }
@@ -147,14 +200,20 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     startProfileIndexChanged(ui->startProfile->currentText());
 
     //APPEREANCE
-    m_themesManager = new ThemeManager(ui->themesWidget, this);
     settings.beginGroup("Browser-View-Settings");
     ui->showStatusbar->setChecked(settings.value("showStatusBar", true).toBool());
-    ui->showBookmarksToolbar->setChecked(p_QupZilla->bookmarksToolbar()->isVisible());
-    ui->showNavigationToolbar->setChecked(p_QupZilla->navigationBar()->isVisible());
+    if (p_QupZilla) {
+        ui->showBookmarksToolbar->setChecked(p_QupZilla->bookmarksToolbar()->isVisible());
+        ui->showNavigationToolbar->setChecked(p_QupZilla->navigationBar()->isVisible());
+    }
+    else {
+        ui->showBookmarksToolbar->setChecked(settings.value("showBookmarksToolbar", true).toBool());
+        ui->showNavigationToolbar->setChecked(settings.value("showNavigationToolbar", true).toBool());
+    }
     ui->showHome->setChecked(settings.value("showHomeButton", true).toBool());
     ui->showBackForward->setChecked(settings.value("showBackForwardButtons", true).toBool());
     ui->showAddTabButton->setChecked(settings.value("showAddTabButton", false).toBool());
+    ui->showReloadStopButtons->setChecked(settings.value("showReloadButton", true).toBool());
     ui->showWebSearchBar->setChecked(settings.value("showWebSearchBar", true).toBool());
     ui->useTransparentBg->setChecked(settings.value("useTransparentBackground", false).toBool());
     int currentSettingsPage = settings.value("settingsDialogPage", 0).toInt(0);
@@ -168,6 +227,8 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     ui->hideTabsOnTab->setChecked(settings.value("hideTabsWithOneTab", false).toBool());
     ui->activateLastTab->setChecked(settings.value("ActivateLastTabWhenClosingActual", false).toBool());
     ui->openNewTabAfterActive->setChecked(settings.value("newTabAfterActive", true).toBool());
+    ui->openNewEmptyTabAfterActive->setChecked(settings.value("newEmptyTabAfterActive", false).toBool());
+    ui->alwaysSwitchTabsWithWheel->setChecked(settings.value("AlwaysSwitchTabsWithWheel", false).toBool());
     ui->switchToNewTabs->setChecked(settings.value("OpenNewTabsSelected", false).toBool());
     ui->dontQuitOnTab->setChecked(settings.value("dontQuitWithOneTab", false).toBool());
     ui->askWhenClosingMultipleTabs->setChecked(settings.value("AskOnClosing", false).toBool());
@@ -182,10 +243,27 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     //AddressBar
     settings.beginGroup("AddressBar");
     ui->addressbarCompletion->setCurrentIndex(settings.value("showSuggestions", 0).toInt());
+    ui->useInlineCompletion->setChecked(settings.value("useInlineCompletion", true).toBool());
+    ui->completionShowSwitchTab->setChecked(settings.value("showSwitchTab", true).toBool());
+    ui->alwaysShowGoIcon->setChecked(settings.value("alwaysShowGoIcon", false).toBool());
     ui->selectAllOnFocus->setChecked(settings.value("SelectAllTextOnDoubleClick", true).toBool());
     ui->selectAllOnClick->setChecked(settings.value("SelectAllTextOnClick", false).toBool());
     ui->addCountryWithAlt->setChecked(settings.value("AddCountryDomainWithAltKey", true).toBool());
-    ui->showLoadingInAddressBar->setChecked(settings.value("ShowLoadingProgress", false).toBool());
+    bool showPBinAB = settings.value("ShowLoadingProgress", false).toBool();
+    ui->showLoadingInAddressBar->setChecked(showPBinAB);
+    ui->adressProgressSettings->setEnabled(showPBinAB);
+    ui->progressStyleSelector->setCurrentIndex(settings.value("ProgressStyle", 0).toInt());
+    bool pbInABuseCC = settings.value("UseCustomProgressColor", false).toBool();
+    ui->checkBoxCustomProgressColor->setChecked(pbInABuseCC);
+    ui->progressBarColorSelector->setEnabled(pbInABuseCC);
+    QColor pbColor = settings.value("CustomProgressColor", p_QupZilla->palette().color(QPalette::Highlight)).value<QColor>();
+    setProgressBarColorIcon(pbColor);
+    connect(ui->customColorToolButton, SIGNAL(clicked(bool)), SLOT(selectCustomProgressBarColor()));
+    connect(ui->resetProgressBarcolor, SIGNAL(clicked()), SLOT(setProgressBarColorIcon()));
+    settings.endGroup();
+
+    settings.beginGroup("SearchEngines");
+    ui->searchWithDefaultEngine->setChecked(settings.value("SearchWithDefaultEngine", false).toBool());
     settings.endGroup();
 
     //BROWSING
@@ -196,15 +274,15 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     ui->allowDNSPrefetch->setChecked(settings.value("DNS-Prefetch", false).toBool());
     ui->linksInFocusChain->setChecked(settings.value("IncludeLinkInFocusChain", false).toBool());
     ui->zoomTextOnly->setChecked(settings.value("zoomTextOnly", false).toBool());
+    ui->spatialNavigation->setChecked(settings.value("SpatialNavigation", false).toBool());
+    ui->caretBrowsing->setChecked(settings.value("CaretBrowsing", false).toBool());
+    ui->animateScrolling->setChecked(settings.value("AnimateScrolling", true).toBool());
     ui->printEBackground->setChecked(settings.value("PrintElementBackground", true).toBool());
     ui->wheelScroll->setValue(settings.value("wheelScrollLines", qApp->wheelScrollLines()).toInt());
     ui->defaultZoom->setValue(settings.value("DefaultZoom", 100).toInt());
     ui->xssAuditing->setChecked(settings.value("XSSAuditing", false).toBool());
+    ui->formsUndoRedo->setChecked(settings.value("enableFormsUndoRedo", false).toBool());
 
-    if (!ui->allowJavaScript->isChecked()) {
-        ui->blockPopup->setEnabled(false);
-    }
-    connect(ui->allowJavaScript, SIGNAL(toggled(bool)), this, SLOT(allowJavaScriptChanged(bool)));
     //Cache
     ui->pagesInCache->setValue(settings.value("maximumCachedPages", 3).toInt());
     connect(ui->pagesInCache, SIGNAL(valueChanged(int)), this, SLOT(pageCacheValueChanged(int)));
@@ -213,16 +291,16 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     ui->allowCache->setChecked(settings.value("AllowLocalCache", true).toBool());
     ui->cacheMB->setValue(settings.value("LocalCacheSize", 50).toInt());
     ui->MBlabel->setText(settings.value("LocalCacheSize", 50).toString() + " MB");
+    ui->cachePath->setText(settings.value("CachePath", QString("%1networkcache/").arg(mApp->currentProfilePath())).toString());
     connect(ui->allowCache, SIGNAL(clicked(bool)), this, SLOT(allowCacheChanged(bool)));
     connect(ui->cacheMB, SIGNAL(valueChanged(int)), this, SLOT(cacheValueChanged(int)));
+    connect(ui->changeCachePath, SIGNAL(clicked()), this, SLOT(changeCachePathClicked()));
     allowCacheChanged(ui->allowCache->isChecked());
 
     //PASSWORD MANAGER
     ui->allowPassManager->setChecked(settings.value("SavePasswordsOnSites", true).toBool());
     connect(ui->allowPassManager, SIGNAL(toggled(bool)), this, SLOT(showPassManager(bool)));
 
-    m_autoFillManager = new AutoFillManager(this);
-    ui->autoFillFrame->addWidget(m_autoFillManager);
     showPassManager(ui->allowPassManager->isChecked());
 
     //PRIVACY
@@ -239,8 +317,6 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     ui->deleteHtml5storageOnClose->setChecked(settings.value("deleteHTML5StorageOnClose", false).toBool());
     connect(ui->html5storage, SIGNAL(toggled(bool)), this, SLOT(allowHtml5storageChanged(bool)));
     // Other
-    ui->blockPopup->setChecked(!settings.value("allowJavaScriptOpenWindow", false).toBool());
-    ui->jscanAccessClipboard->setChecked(settings.value("JavaScriptCanAccessClipboard", true).toBool());
     ui->doNotTrack->setChecked(settings.value("DoNotTrack", false).toBool());
     ui->sendReferer->setChecked(settings.value("SendReferer", true).toBool());
 
@@ -249,18 +325,6 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     connect(ui->chooseUserStylesheet, SIGNAL(clicked()), this, SLOT(chooseUserStyleClicked()));
     settings.endGroup();
 
-    //Cookies
-    settings.beginGroup("Cookie-Settings");
-    ui->saveCookies->setChecked(settings.value("allowCookies", true).toBool());
-    if (!ui->saveCookies->isChecked()) {
-        ui->deleteCookiesOnClose->setEnabled(false);
-    }
-    connect(ui->saveCookies, SIGNAL(toggled(bool)), this, SLOT(saveCookiesChanged(bool)));
-    ui->deleteCookiesOnClose->setChecked(settings.value("deleteCookiesOnClose", false).toBool());
-    ui->matchExactly->setChecked(settings.value("allowCookiesFromVisitedDomainOnly", false).toBool());
-    ui->filterTracking->setChecked(settings.value("filterTrackingCookie", false).toBool());
-    settings.endGroup();
-
     //DOWNLOADS
     settings.beginGroup("DownloadManager");
     ui->downLoc->setText(settings.value("defaultDownloadPath", "").toString());
@@ -299,9 +363,11 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     ui->sizeMinimumLogical->setValue(settings.value("MinimumLogicalFontSize", mApp->webSettings()->fontSize(QWebSettings::MinimumLogicalFontSize)).toInt());
     settings.endGroup();
 
-    //PLUGINS
-    m_pluginsList = new PluginsManager(this);
-    ui->pluginsFrame->addWidget(m_pluginsList);
+    //KEYBOARD SHORTCUTS
+    settings.beginGroup("Shortcuts");
+    ui->switchTabsAlt->setChecked(settings.value("useTabNumberShortcuts", true).toBool());
+    ui->loadSpeedDialsCtrl->setChecked(settings.value("useSpeedDialNumberShortcuts", true).toBool());
+    settings.endGroup();
 
     //NOTIFICATIONS
     ui->useNativeSystemNotifications->setEnabled(mApp->desktopNotifications()->supportsNativeNotifications());
@@ -309,7 +375,7 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     DesktopNotificationsFactory::Type notifyType;
     settings.beginGroup("Notifications");
     ui->notificationTimeout->setValue(settings.value("Timeout", 6000).toInt() / 1000);
-#ifdef Q_WS_X11
+#ifdef QZ_WS_X11
     notifyType = settings.value("UseNativeDesktop", true).toBool() ? DesktopNotificationsFactory::DesktopNative : DesktopNotificationsFactory::PopupWidget;
 #else
     notifyType = DesktopNotificationsFactory::PopupWidget;
@@ -330,19 +396,17 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
 
     //OTHER
     //Languages
-    QString activeLanguage;
-    if (!mApp->currentLanguage().isEmpty()) {
-        activeLanguage = mApp->currentLanguage();
-        QLocale locale(activeLanguage);
-        QString country = QLocale::countryToString(locale.country());
-        QString language = QLocale::languageToString(locale.language());
-        ui->languages->addItem(language + ", " + country + " (" + activeLanguage + ")", activeLanguage);
+    QString activeLanguage = mApp->currentLanguage();
+
+    if (!activeLanguage.isEmpty() && activeLanguage != QLatin1String("en_US")) {
+        ui->languages->addItem(createLanguageItem(activeLanguage), activeLanguage);
     }
+
     ui->languages->addItem("English (en_US)");
 
     QDir lanDir(mApp->TRANSLATIONSDIR);
     QStringList list = lanDir.entryList(QStringList("*.qm"));
-    foreach(const QString & name, list) {
+    foreach (const QString &name, list) {
         if (name.startsWith(QLatin1String("qt_"))) {
             continue;
         }
@@ -354,10 +418,7 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
             continue;
         }
 
-        QLocale locale(loc);
-        QString country = QLocale::countryToString(locale.country());
-        QString language = QLocale::languageToString(locale.language());
-        ui->languages->addItem(language + ", " + country + " (" + loc + ")", loc);
+        ui->languages->addItem(createLanguageItem(loc), loc);
     }
 
     // Proxy Configuration
@@ -368,6 +429,7 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     ui->systemProxy->setChecked(proxyPreference == NetworkProxyFactory::SystemProxy);
     ui->noProxy->setChecked(proxyPreference == NetworkProxyFactory::NoProxy);
     ui->manualProxy->setChecked(proxyPreference == NetworkProxyFactory::DefinedProxy);
+    ui->pacProxy->setChecked(proxyPreference == NetworkProxyFactory::ProxyAutoConfig);
     if (proxyType == QNetworkProxy::HttpProxy) {
         ui->proxyType->setCurrentIndex(0);
     }
@@ -386,28 +448,55 @@ Preferences::Preferences(QupZilla* mainClass, QWidget* parent)
     ui->httpsProxyUsername->setText(settings.value("HttpsUsername", "").toString());
     ui->httpsProxyPassword->setText(settings.value("HttpsPassword", "").toString());
 
+    ui->pacUrl->setText(settings.value("PacUrl", QUrl()).toUrl().toString());
     ui->proxyExceptions->setText(settings.value("ProxyExceptions", QStringList() << "localhost" << "127.0.0.1").toStringList().join(","));
     settings.endGroup();
 
     useDifferentProxyForHttpsChanged(ui->useHttpsProxy->isChecked());
     setManualProxyConfigurationEnabled(proxyPreference == NetworkProxyFactory::DefinedProxy);
+    setProxyAutoConfigEnabled(proxyPreference == NetworkProxyFactory::ProxyAutoConfig);
 
     connect(ui->manualProxy, SIGNAL(toggled(bool)), this, SLOT(setManualProxyConfigurationEnabled(bool)));
+    connect(ui->pacProxy, SIGNAL(toggled(bool)), this, SLOT(setProxyAutoConfigEnabled(bool)));
     connect(ui->useHttpsProxy, SIGNAL(toggled(bool)), this, SLOT(useDifferentProxyForHttpsChanged(bool)));
+    connect(ui->reloadPac, SIGNAL(clicked()), this, SLOT(reloadPacFileClicked()));
 
     //CONNECTS
     connect(ui->buttonBox, SIGNAL(clicked(QAbstractButton*)), this, SLOT(buttonClicked(QAbstractButton*)));
     connect(ui->cookieManagerBut, SIGNAL(clicked()), this, SLOT(showCookieManager()));
+    connect(ui->html5permissions, SIGNAL(clicked()), this, SLOT(showHtml5Permissions()));
     connect(ui->sslManagerButton, SIGNAL(clicked()), this, SLOT(openSslManager()));
     connect(ui->preferredLanguages, SIGNAL(clicked()), this, SLOT(showAcceptLanguage()));
     connect(ui->deleteHtml5storage, SIGNAL(clicked()), this, SLOT(deleteHtml5storage()));
     connect(ui->uaManager, SIGNAL(clicked()), this, SLOT(openUserAgentManager()));
+    connect(ui->jsOptionsButton, SIGNAL(clicked()), this, SLOT(openJsOptions()));
 
-    connect(ui->listWidget, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)), this, SLOT(showStackedPage(QListWidgetItem*)));
+    connect(ui->listWidget, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)), this, SLOT(showStackedPage(QListWidgetItem*)));
     ui->listWidget->setItemSelected(ui->listWidget->itemAt(5, 5), true);
 
     ui->version->setText(" QupZilla v" + QupZilla::VERSION);
     ui->listWidget->setCurrentRow(currentSettingsPage);
+
+    QDesktopWidget* desktop = QApplication::desktop();
+    QSize s = size();
+    if (desktop->availableGeometry(this).size().width() < s.width()) {
+        s.setWidth(desktop->availableGeometry(this).size().width() - 50);
+    }
+    if (desktop->availableGeometry(this).size().height() < s.height()) {
+        s.setHeight(desktop->availableGeometry(this).size().height() - 50);
+    }
+    resize(s);
+
+#if QTWEBKIT_TO_2_3
+    ui->caretBrowsing->setHidden(true);
+    ui->animateScrolling->setHidden(true);
+#endif
+
+#if QTWEBKIT_TO_2_2
+    ui->html5permissions->setDisabled(true);
+#endif
+
+    QzTools::setWmClass("Preferences", this);
 }
 
 void Preferences::showStackedPage(QListWidgetItem* item)
@@ -421,10 +510,17 @@ void Preferences::showStackedPage(QListWidgetItem* item)
     ui->caption->setText("<b>" + item->text() + "</b>");
     ui->stackedWidget->setCurrentIndex(index);
 
-    setNotificationPreviewVisible(index == 8);
-    if (index == 9) {
+    setNotificationPreviewVisible(index == 9);
+
+    if (index == 10) {
         m_pluginsList->load();
     }
+
+    if (index == 7 && !m_autoFillManager) {
+        m_autoFillManager = new AutoFillManager(this);
+        ui->autoFillFrame->addWidget(m_autoFillManager);
+        m_autoFillManager->setVisible(m_autoFillEnabled);
+    }
 }
 
 void Preferences::setNotificationPreviewVisible(bool state)
@@ -454,10 +550,23 @@ void Preferences::setNotificationPreviewVisible(bool state)
     }
 }
 
+void Preferences::makeQupZillaDefault()
+{
+#if defined(Q_OS_WIN) && !defined(Q_OS_OS2)
+    disconnect(ui->checkNowDefaultBrowser, SIGNAL(clicked()), this, SLOT(makeQupZillaDefault()));
+    mApp->associationManager()->registerAllAssociation();
+    ui->checkNowDefaultBrowser->setText(tr("Default"));
+    ui->checkNowDefaultBrowser->setEnabled(false);
+#endif
+}
+
 void Preferences::allowCacheChanged(bool state)
 {
     ui->cacheFrame->setEnabled(state);
     ui->cacheMB->setEnabled(state);
+    ui->storeCacheLabel->setEnabled(state);
+    ui->cachePath->setEnabled(state);
+    ui->changeCachePath->setEnabled(state);
 }
 
 void Preferences::useActualHomepage()
@@ -472,7 +581,7 @@ void Preferences::useActualNewTab()
 
 void Preferences::chooseDownPath()
 {
-    QString userFileName = QFileDialog::getExistingDirectory(p_QupZilla, tr("Choose download location..."), QDir::homePath());
+    QString userFileName = QzTools::getExistingDirectory("Preferences-ChooseDownPath", this, tr("Choose download location..."), QDir::homePath());
     if (userFileName.isEmpty()) {
         return;
     }
@@ -486,7 +595,7 @@ void Preferences::chooseDownPath()
 
 void Preferences::chooseUserStyleClicked()
 {
-    QString file = QFileDialog::getOpenFileName(p_QupZilla, tr("Choose stylesheet location..."), QDir::homePath(), "*.css");
+    QString file = QzTools::getOpenFileName("Preferences-UserStyle", this, tr("Choose stylesheet location..."), QDir::homePath(), "*.css");
     if (file.isEmpty()) {
         return;
     }
@@ -503,7 +612,7 @@ void Preferences::deleteHtml5storage()
 
 void Preferences::chooseExternalDownloadManager()
 {
-    QString path = QFileDialog::getOpenFileName(p_QupZilla, tr("Choose executable location..."), QDir::homePath());
+    QString path = QzTools::getOpenFileName("Preferences-ExternalDownloadManager", this, tr("Choose executable location..."), QDir::homePath());
     if (path.isEmpty()) {
         return;
     }
@@ -536,9 +645,10 @@ void Preferences::setManualProxyConfigurationEnabled(bool state)
     ui->useHttpsProxy->setEnabled(state);
 }
 
-void Preferences::allowJavaScriptChanged(bool state)
+void Preferences::setProxyAutoConfigEnabled(bool state)
 {
-    ui->blockPopup->setEnabled(state);
+    ui->pacUrl->setEnabled(state);
+    ui->reloadPac->setEnabled(state);
 }
 
 void Preferences::saveHistoryChanged(bool stat)
@@ -546,11 +656,6 @@ void Preferences::saveHistoryChanged(bool stat)
     ui->deleteHistoryOnClose->setEnabled(stat);
 }
 
-void Preferences::saveCookiesChanged(bool state)
-{
-    ui->deleteCookiesOnClose->setEnabled(state);
-}
-
 void Preferences::allowHtml5storageChanged(bool stat)
 {
     ui->deleteHtml5storageOnClose->setEnabled(stat);
@@ -565,12 +670,24 @@ void Preferences::showCookieManager()
     m->raise();
 }
 
+void Preferences::showHtml5Permissions()
+{
+    HTML5PermissionsDialog dialog(this);
+    dialog.exec();
+}
+
 void Preferences::openSslManager()
 {
     SSLManager* m = new SSLManager(this);
     m->show();
 }
 
+void Preferences::openJsOptions()
+{
+    JsOptions options(this);
+    options.exec();
+}
+
 void Preferences::showAcceptLanguage()
 {
     AcceptLanguage a(this);
@@ -590,14 +707,6 @@ void Preferences::afterLaunchChanged(int value)
 void Preferences::cacheValueChanged(int value)
 {
     ui->MBlabel->setText(QString::number(value) + " MB");
-    if (value == 0) {
-        ui->allowCache->setChecked(false);
-        allowCacheChanged(false);
-    }
-    else if (!ui->allowCache->isChecked()) {
-        ui->allowCache->setChecked(true);
-        allowCacheChanged(true);
-    }
 }
 
 void Preferences::pageCacheValueChanged(int value)
@@ -625,9 +734,29 @@ void Preferences::showTabPreviewsChanged(bool state)
     ui->animatedTabPreviews->setEnabled(state);
 }
 
+void Preferences::changeCachePathClicked()
+{
+    QString path = QzTools::getExistingDirectory("Preferences-CachePath", this, tr("Choose cache path..."), ui->cachePath->text());
+    if (path.isEmpty()) {
+        return;
+    }
+
+    ui->cachePath->setText(path);
+}
+
+void Preferences::reloadPacFileClicked()
+{
+    mApp->networkManager()->proxyFactory()->pacManager()->downloadPacFile();
+}
+
 void Preferences::showPassManager(bool state)
 {
-    m_autoFillManager->setVisible(state);
+    if (m_autoFillManager) {
+        m_autoFillManager->setVisible(state);
+    }
+    else {
+        m_autoFillEnabled = state;
+    }
 }
 
 void Preferences::buttonClicked(QAbstractButton* button)
@@ -654,7 +783,7 @@ void Preferences::buttonClicked(QAbstractButton* button)
 void Preferences::createProfile()
 {
     QString name = QInputDialog::getText(this, tr("New Profile"), tr("Enter the new profile's name:"));
-    name = qz_filterCharsFromFilename(name);
+    name = QzTools::filterCharsFromFilename(name);
     if (name.isEmpty()) {
         return;
     }
@@ -690,7 +819,7 @@ void Preferences::deleteProfile()
         return;
     }
 
-    qz_removeDir(mApp->PROFILEDIR + "profiles/" + name);
+    QzTools::removeDir(mApp->PROFILEDIR + "profiles/" + name);
     ui->startProfile->removeItem(ui->startProfile->currentIndex());
 }
 
@@ -728,7 +857,7 @@ void Preferences::saveSettings()
 
     switch (ui->newTab->currentIndex()) {
     case 0:
-        settings.setValue("newTabUrl", "");
+        settings.setValue("newTabUrl", QString());
         break;
 
     case 1:
@@ -765,6 +894,7 @@ void Preferences::saveSettings()
     settings.setValue("showWebSearchBar", ui->showWebSearchBar->isChecked());
     settings.setValue("useTransparentBackground", ui->useTransparentBg->isChecked());
     settings.setValue("showAddTabButton", ui->showAddTabButton->isChecked());
+    settings.setValue("showReloadButton", ui->showReloadStopButtons->isChecked());
     settings.endGroup();
 
     //TABS
@@ -772,6 +902,8 @@ void Preferences::saveSettings()
     settings.setValue("hideTabsWithOneTab", ui->hideTabsOnTab->isChecked());
     settings.setValue("ActivateLastTabWhenClosingActual", ui->activateLastTab->isChecked());
     settings.setValue("newTabAfterActive", ui->openNewTabAfterActive->isChecked());
+    settings.setValue("newEmptyTabAfterActive", ui->openNewEmptyTabAfterActive->isChecked());
+    settings.setValue("AlwaysSwitchTabsWithWheel", ui->alwaysSwitchTabsWithWheel->isChecked());
     settings.setValue("OpenNewTabsSelected", ui->switchToNewTabs->isChecked());
     settings.setValue("dontQuitWithOneTab", ui->dontQuitOnTab->isChecked());
     settings.setValue("AskOnClosing", ui->askWhenClosingMultipleTabs->isChecked());
@@ -798,6 +930,7 @@ void Preferences::saveSettings()
     //FONTS
     settings.beginGroup("Browser-Fonts");
     settings.setValue("StandardFont", ui->fontStandard->currentFont().family());
+    settings.setValue("CursiveFont", ui->fontCursive->currentFont().family());
     settings.setValue("FantasyFont", ui->fontFantasy->currentFont().family());
     settings.setValue("FixedFont", ui->fontFixed->currentFont().family());
     settings.setValue("SansSerifFont", ui->fontSansSerif->currentFont().family());
@@ -809,16 +942,23 @@ void Preferences::saveSettings()
     settings.setValue("MinimumLogicalFontSize", ui->sizeMinimumLogical->value());
     settings.endGroup();
 
+    //KEYBOARD SHORTCUTS
+    settings.beginGroup("Shortcuts");
+    settings.setValue("useTabNumberShortcuts", ui->switchTabsAlt->isChecked());
+    settings.setValue("useSpeedDialNumberShortcuts", ui->loadSpeedDialsCtrl->isChecked());
+    settings.endGroup();
+
     //BROWSING
     settings.beginGroup("Web-Browser-Settings");
     settings.setValue("allowFlash", ui->allowPlugins->isChecked());
     settings.setValue("allowJavaScript", ui->allowJavaScript->isChecked());
-    settings.setValue("allowJavaScriptOpenWindow", !ui->blockPopup->isChecked());
     settings.setValue("allowJava", ui->allowJava->isChecked());
     settings.setValue("DNS-Prefetch", ui->allowDNSPrefetch->isChecked());
-    settings.setValue("JavaScriptCanAccessClipboard", ui->jscanAccessClipboard->isChecked());
     settings.setValue("IncludeLinkInFocusChain", ui->linksInFocusChain->isChecked());
     settings.setValue("zoomTextOnly", ui->zoomTextOnly->isChecked());
+    settings.setValue("SpatialNavigation", ui->spatialNavigation->isChecked());
+    settings.setValue("CaretBrowsing", ui->caretBrowsing->isChecked());
+    settings.setValue("AnimateScrolling", ui->animateScrolling->isChecked());
     settings.setValue("PrintElementBackground", ui->printEBackground->isChecked());
     settings.setValue("wheelScrollLines", ui->wheelScroll->value());
     settings.setValue("DoNotTrack", ui->doNotTrack->isChecked());
@@ -826,11 +966,15 @@ void Preferences::saveSettings()
     settings.setValue("LoadTabsOnActivation", ui->dontLoadTabsUntilSelected->isChecked());
     settings.setValue("DefaultZoom", ui->defaultZoom->value());
     settings.setValue("XSSAuditing", ui->xssAuditing->isChecked());
-
+    settings.setValue("enableFormsUndoRedo", ui->formsUndoRedo->isChecked());
+#ifdef Q_OS_WIN
+    settings.setValue("CheckDefaultBrowser", ui->checkDefaultBrowser->isChecked());
+#endif
     //Cache
     settings.setValue("maximumCachedPages", ui->pagesInCache->value());
     settings.setValue("AllowLocalCache", ui->allowCache->isChecked());
     settings.setValue("LocalCacheSize", ui->cacheMB->value());
+    settings.setValue("CachePath", ui->cachePath->text());
     //CSS Style
     settings.setValue("userStyleSheet", ui->userStyleSheet->text());
 
@@ -846,14 +990,6 @@ void Preferences::saveSettings()
     settings.setValue("SendReferer", ui->sendReferer->isChecked());
     settings.endGroup();
 
-    //Cookies
-    settings.beginGroup("Cookie-Settings");
-    settings.setValue("allowCookies", ui->saveCookies->isChecked());
-    settings.setValue("deleteCookiesOnClose", ui->deleteCookiesOnClose->isChecked());
-    settings.setValue("allowCookiesFromVisitedDomainOnly", ui->matchExactly->isChecked());
-    settings.setValue("filterTrackingCookie", ui->filterTracking->isChecked());
-    settings.endGroup();
-
     //NOTIFICATIONS
     settings.beginGroup("Notifications");
     settings.setValue("Timeout", ui->notificationTimeout->value() * 1000);
@@ -866,10 +1002,20 @@ void Preferences::saveSettings()
     //AddressBar
     settings.beginGroup("AddressBar");
     settings.setValue("showSuggestions", ui->addressbarCompletion->currentIndex());
+    settings.setValue("useInlineCompletion", ui->useInlineCompletion->isChecked());
+    settings.setValue("alwaysShowGoIcon", ui->alwaysShowGoIcon->isChecked());
+    settings.setValue("showSwitchTab", ui->completionShowSwitchTab->isChecked());
     settings.setValue("SelectAllTextOnDoubleClick", ui->selectAllOnFocus->isChecked());
     settings.setValue("SelectAllTextOnClick", ui->selectAllOnClick->isChecked());
     settings.setValue("AddCountryDomainWithAltKey", ui->addCountryWithAlt->isChecked());
     settings.setValue("ShowLoadingProgress", ui->showLoadingInAddressBar->isChecked());
+    settings.setValue("ProgressStyle", ui->progressStyleSelector->currentIndex());
+    settings.setValue("UseCustomProgressColor", ui->checkBoxCustomProgressColor->isChecked());
+    settings.setValue("CustomProgressColor", ui->customColorToolButton->property("ProgressColor").value<QColor>());
+    settings.endGroup();
+
+    settings.beginGroup("SearchEngines");
+    settings.setValue("SearchWithDefaultEngine", ui->searchWithDefaultEngine->isChecked());
     settings.endGroup();
 
     //Languages
@@ -885,6 +1031,9 @@ void Preferences::saveSettings()
     else if (ui->noProxy->isChecked()) {
         proxyPreference = NetworkProxyFactory::NoProxy;
     }
+    else if (ui->pacProxy->isChecked()) {
+        proxyPreference = NetworkProxyFactory::ProxyAutoConfig;
+    }
     else {
         proxyPreference = NetworkProxyFactory::DefinedProxy;
     }
@@ -911,7 +1060,8 @@ void Preferences::saveSettings()
     settings.setValue("HttpsUsername", ui->httpsProxyUsername->text());
     settings.setValue("HttpsPassword", ui->httpsProxyPassword->text());
 
-    settings.setValue("ProxyExceptions", ui->proxyExceptions->text().split(QLatin1Char(',')));
+    settings.setValue("PacUrl", ui->pacUrl->text());
+    settings.setValue("ProxyExceptions", ui->proxyExceptions->text().split(QLatin1Char(','), QString::SkipEmptyParts));
     settings.endGroup();
 
     //Profiles
@@ -936,3 +1086,23 @@ Preferences::~Preferences()
     delete m_pluginsList;
     delete m_notification.data();
 }
+
+void Preferences::setProgressBarColorIcon(QColor color)
+{
+    const int size = style()->pixelMetric(QStyle::PM_ToolBarIconSize);
+    QPixmap pm(QSize(size, size));
+    if (!color.isValid()) {
+        color = p_QupZilla->palette().color(QPalette::Highlight);
+    }
+    pm.fill(color);
+    ui->customColorToolButton->setIcon(pm);
+    ui->customColorToolButton->setProperty("ProgressColor", color);
+}
+
+void Preferences::selectCustomProgressBarColor()
+{
+    QColor newColor = QColorDialog::getColor(ui->customColorToolButton->property("ProgressColor").value<QColor>(), this, tr("Select Color"));
+    if (newColor.isValid()) {
+        setProgressBarColorIcon(newColor);
+    }
+}
