@@ -1,6 +1,6 @@
 /* ============================================================
 * QupZilla - WebKit based browser
-* Copyright (C) 2010-2012  David Rosca <nowrep@gmail.com>
+* Copyright (C) 2010-2013  David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -30,17 +30,21 @@
 #include "locationbar.h"
 #include "websearchbar.h"
 #include "settings.h"
+#include "qzsettings.h"
+#include "qtwin.h"
 
+#include <QTimer>
 #include <QMovie>
-#include <QMenu>
+#include <QMimeData>
 #include <QStackedWidget>
 #include <QMouseEvent>
 #include <QWebHistory>
 #include <QClipboard>
 #include <QFile>
+#include <QScrollArea>
 
 AddTabButton::AddTabButton(TabWidget* tabWidget, TabBar* tabBar)
-    : ToolButton(tabWidget)
+    : ToolButton(tabBar)
     , m_tabBar(tabBar)
     , m_tabWidget(tabWidget)
 {
@@ -59,12 +63,7 @@ void AddTabButton::wheelEvent(QWheelEvent* event)
 void AddTabButton::mouseReleaseEvent(QMouseEvent* event)
 {
     if (event->button() == Qt::MiddleButton && rect().contains(event->pos())) {
-        QString selectionClipboard = QApplication::clipboard()->text(QClipboard::Selection);
-        QUrl guessedUrl = WebView::guessUrlFromString(selectionClipboard);
-
-        if (!guessedUrl.isEmpty()) {
-            m_tabWidget->addView(guessedUrl, Qz::NT_SelectedTabAtTheEnd);
-        }
+        m_tabWidget->addTabFromClipboard();
     }
 
     ToolButton::mouseReleaseEvent(event);
@@ -91,15 +90,31 @@ void AddTabButton::dropEvent(QDropEvent* event)
         return;
     }
 
-    foreach(const QUrl & url, mime->urls()) {
-        m_tabWidget->addView(url, Qz::NT_SelectedTabAtTheEnd);
+    foreach (const QUrl &url, mime->urls()) {
+        m_tabWidget->addView(url, Qz::NT_SelectedNewEmptyTab);
     }
 }
 
+void MenuTabs::mouseReleaseEvent(QMouseEvent* event)
+{
+    if (event->button() == Qt::MiddleButton) {
+        QAction* action = actionAt(event->pos());
+        if (action && action->isEnabled()) {
+            WebTab* tab = qobject_cast<WebTab*>(qvariant_cast<QWidget*>(action->data()));
+            if (tab) {
+                emit closeTab(tab->tabIndex());
+                action->setEnabled(false);
+                event->accept();
+            }
+        }
+    }
+    QMenu::mouseReleaseEvent(event);
+}
+
 TabWidget::TabWidget(QupZilla* mainClass, QWidget* parent)
-    : QTabWidget(parent)
+    : TabStackedWidget(parent)
     , p_QupZilla(mainClass)
-    , m_lastTabIndex(0)
+    , m_lastTabIndex(-1)
     , m_lastBackgroundTabIndex(-1)
     , m_isClosingToLastTabIndex(false)
     , m_isRestoringState(false)
@@ -110,7 +125,6 @@ TabWidget::TabWidget(QupZilla* mainClass, QWidget* parent)
     m_tabBar = new TabBar(p_QupZilla, this);
     setTabBar(m_tabBar);
 
-    connect(this, SIGNAL(currentChanged(int)), this, SLOT(currentTabChanged(int)));
     connect(this, SIGNAL(currentChanged(int)), p_QupZilla, SLOT(refreshHistory()));
 
     connect(m_tabBar, SIGNAL(tabCloseRequested(int)), this, SLOT(closeTab(int)));
@@ -119,15 +133,16 @@ TabWidget::TabWidget(QupZilla* mainClass, QWidget* parent)
     connect(m_tabBar, SIGNAL(closeTab(int)), this, SLOT(closeTab(int)));
     connect(m_tabBar, SIGNAL(closeAllButCurrent(int)), this, SLOT(closeAllButCurrent(int)));
     connect(m_tabBar, SIGNAL(duplicateTab(int)), this, SLOT(duplicateTab(int)));
-    connect(m_tabBar, SIGNAL(tabMoved(int, int)), this, SLOT(tabMoved(int, int)));
+    connect(m_tabBar, SIGNAL(detachTab(int)), this, SLOT(detachTab(int)));
+    connect(m_tabBar, SIGNAL(tabMoved(int,int)), this, SLOT(tabMoved(int,int)));
 
     connect(m_tabBar, SIGNAL(moveAddTabButton(int)), this, SLOT(moveAddTabButton(int)));
     connect(m_tabBar, SIGNAL(showButtons()), this, SLOT(showButtons()));
     connect(m_tabBar, SIGNAL(hideButtons()), this, SLOT(hideButtons()));
 
-    m_buttonListTabs = new ToolButton(this);
+    m_buttonListTabs = new ToolButton(m_tabBar);
     m_buttonListTabs->setObjectName("tabwidget-button-opentabs");
-    m_menuTabs = new QMenu(this);
+    m_menuTabs = new MenuTabs(m_tabBar);
     m_buttonListTabs->setMenu(m_menuTabs);
     m_buttonListTabs->setPopupMode(QToolButton::InstantPopup);
     m_buttonListTabs->setToolTip(tr("List of tabs"));
@@ -139,6 +154,26 @@ TabWidget::TabWidget(QupZilla* mainClass, QWidget* parent)
     connect(m_buttonAddTab, SIGNAL(clicked()), p_QupZilla, SLOT(addTab()));
     connect(m_menuTabs, SIGNAL(aboutToShow()), this, SLOT(aboutToShowClosedTabsMenu()));
 
+    // Copy of buttons
+    m_buttonListTabs2 = new ToolButton(m_tabBar);
+    m_buttonListTabs2->setObjectName("tabwidget-button-opentabs");
+    m_buttonListTabs2->setProperty("outside-tabbar", true);
+    m_buttonListTabs2->setMenu(m_menuTabs);
+    m_buttonListTabs2->setPopupMode(QToolButton::InstantPopup);
+    m_buttonListTabs2->setToolTip(tr("List of tabs"));
+    m_buttonListTabs2->setAutoRaise(true);
+    m_buttonListTabs2->setFocusPolicy(Qt::NoFocus);
+
+    m_buttonAddTab2 = new AddTabButton(this, m_tabBar);
+    m_buttonAddTab2->setProperty("outside-tabbar", true);
+    connect(m_buttonAddTab2, SIGNAL(clicked()), p_QupZilla, SLOT(addTab()));
+
+    m_tabBar->addMainBarWidget(m_buttonAddTab2, Qt::AlignRight);
+    m_tabBar->addMainBarWidget(m_buttonListTabs2, Qt::AlignRight);
+    m_buttonAddTab2->hide();
+    m_buttonListTabs2->hide();
+    connect(m_tabBar, SIGNAL(overFlowChanged(bool)), this, SLOT(tabBarOverFlowChanged(bool)));
+
     loadSettings();
 }
 
@@ -146,10 +181,10 @@ void TabWidget::loadSettings()
 {
     Settings settings;
     settings.beginGroup("Browser-Tabs-Settings");
-    m_hideTabBarWithOneTab = settings.value("hideTabsWithOneTab", false).toBool();
     m_dontQuitWithOneTab = settings.value("dontQuitWithOneTab", false).toBool();
     m_closedInsteadOpened = settings.value("closedInsteadOpenedTabs", false).toBool();
     m_newTabAfterActive = settings.value("newTabAfterActive", true).toBool();
+    m_newEmptyTabAfterActive = settings.value("newEmptyTabAfterActive", false).toBool();
     settings.endGroup();
 
     settings.beginGroup("Web-URL-Settings");
@@ -157,22 +192,13 @@ void TabWidget::loadSettings()
     settings.endGroup();
 
     m_tabBar->loadSettings();
-}
 
-void TabWidget::resizeEvent(QResizeEvent* e)
-{
-    QPoint posit;
-    posit.setY(0);
-    //RTL Support
-    if (QApplication::layoutDirection() == Qt::RightToLeft) {
-        posit.setX(0);
+    if (m_closedInsteadOpened) {
+        disconnect(m_menuTabs, SIGNAL(closeTab(int)), this, SLOT(closeTab(int)));
     }
     else {
-        posit.setX(width() - m_buttonListTabs->width());
+        connect(m_menuTabs, SIGNAL(closeTab(int)), this, SLOT(closeTab(int)));
     }
-    m_buttonListTabs->move(posit);
-
-    QTabWidget::resizeEvent(e);
 }
 
 WebTab* TabWidget::weTab()
@@ -197,6 +223,12 @@ void TabWidget::hideButtons()
     m_buttonAddTab->hide();
 }
 
+void TabWidget::tabBarOverFlowChanged(bool overFlowed)
+{
+    m_buttonAddTab2->setVisible(overFlowed);
+    m_buttonListTabs2->setVisible(overFlowed);
+}
+
 void TabWidget::moveAddTabButton(int posX)
 {
     int posY = (m_tabBar->height() - m_buttonAddTab->height()) / 2;
@@ -247,39 +279,50 @@ void TabWidget::aboutToShowTabsMenu()
             }
             action->setText(title);
         }
-        action->setData(i);
+        action->setData(QVariant::fromValue(qobject_cast<QWidget*>(tab)));
         connect(action, SIGNAL(triggered()), this, SLOT(actionChangeIndex()));
 
         m_menuTabs->addAction(action);
     }
     m_menuTabs->addSeparator();
-    m_menuTabs->addAction(tr("Currently you have %1 opened tabs").arg(count()))->setEnabled(false);
+    m_menuTabs->addAction(tr("Currently you have %n opened tab(s)", "", count()))->setEnabled(false);
 }
 
 void TabWidget::actionChangeIndex()
 {
     if (QAction* action = qobject_cast<QAction*>(sender())) {
-        setCurrentIndex(action->data().toInt());
+        WebTab* tab = qobject_cast<WebTab*>(qvariant_cast<QWidget*>(action->data()));
+        if (tab) {
+            // needed when clicking on action of the current tab
+            m_tabBar->ensureVisible(tab->tabIndex());
+
+            setCurrentIndex(tab->tabIndex());
+        }
     }
 }
 
-int TabWidget::addView(const QUrl &url, const Qz::NewTabPositionFlags &openFlags, bool selectLine)
+int TabWidget::addView(const QUrl &url, const Qz::NewTabPositionFlags &openFlags, bool selectLine, bool pinned)
 {
-    return addView(QNetworkRequest(url), openFlags, selectLine);
+    return addView(QNetworkRequest(url), openFlags, selectLine, pinned);
 }
 
-int TabWidget::addView(const QNetworkRequest &req, const Qz::NewTabPositionFlags &openFlags, bool selectLine)
+int TabWidget::addView(const QNetworkRequest &req, const Qz::NewTabPositionFlags &openFlags, bool selectLine, bool pinned)
 {
-    return addView(req, tr("New tab"), openFlags, selectLine);
+    return addView(req, tr("New tab"), openFlags, selectLine, -1, pinned);
 }
 
-int TabWidget::addView(const QUrl &url, const QString &title, const Qz::NewTabPositionFlags &openFlags, bool selectLine, int position)
+int TabWidget::addView(const QUrl &url, const QString &title, const Qz::NewTabPositionFlags &openFlags, bool selectLine, int position, bool pinned)
 {
-    return addView(QNetworkRequest(url), title, openFlags, selectLine, position);
+    return addView(QNetworkRequest(url), title, openFlags, selectLine, position, pinned);
 }
 
-int TabWidget::addView(QNetworkRequest req, const QString &title, const Qz::NewTabPositionFlags &openFlags, bool selectLine, int position)
+int TabWidget::addView(QNetworkRequest req, const QString &title, const Qz::NewTabPositionFlags &openFlags, bool selectLine, int position, bool pinned)
 {
+#ifdef Q_OS_WIN
+    if (p_QupZilla->isTransparentBackgroundAllowed()) {
+        QtWin::extendFrameIntoClientArea(p_QupZilla);
+    }
+#endif
     QUrl url = req.url();
     m_lastTabIndex = currentIndex();
 
@@ -287,7 +330,13 @@ int TabWidget::addView(QNetworkRequest req, const QString &title, const Qz::NewT
         url = m_urlOnNewTab;
     }
 
-    if (position == -1 && m_newTabAfterActive && !(openFlags & Qz::NT_TabAtTheEnd)) {
+    bool openAfterActive = m_newTabAfterActive && !(openFlags & Qz::NT_TabAtTheEnd);
+
+    if (openFlags == Qz::NT_SelectedNewEmptyTab && m_newEmptyTabAfterActive) {
+        openAfterActive = true;
+    }
+
+    if (openAfterActive && position == -1) {
         // If we are opening newBgTab from pinned tab, make sure it won't be
         // opened between other pinned tabs
         if (openFlags & Qz::NT_NotSelectedTab && m_lastBackgroundTabIndex != -1) {
@@ -303,10 +352,10 @@ int TabWidget::addView(QNetworkRequest req, const QString &title, const Qz::NewT
     int index;
 
     if (position == -1) {
-        index = addTab(new WebTab(p_QupZilla, locBar), QString());
+        index = addTab(new WebTab(p_QupZilla, locBar), QString(), pinned);
     }
     else {
-        index = insertTab(position, new WebTab(p_QupZilla, locBar), QString());
+        index = insertTab(position, new WebTab(p_QupZilla, locBar), QString(), pinned);
     }
 
     TabbedWebView* webView = weTab(index)->view();
@@ -331,7 +380,7 @@ int TabWidget::addView(QNetworkRequest req, const QString &title, const Qz::NewT
         webView->load(req);
     }
 
-    if (selectLine) {
+    if (selectLine && p_QupZilla->locationBar()->text().isEmpty()) {
         p_QupZilla->locationBar()->setFocus();
     }
 
@@ -339,9 +388,55 @@ int TabWidget::addView(QNetworkRequest req, const QString &title, const Qz::NewT
         m_isClosingToLastTabIndex = true;
     }
 
+    if (openFlags & Qz::NT_NotSelectedTab) {
+        WebTab* currentWebTab = weTab();
+        // Workarounding invalid QWebPage::viewportSize() until QWebView is shown
+        // Fixes invalid scrolling to anchor(#) links
+        if (currentWebTab && currentWebTab->view()) {
+            TabbedWebView* currentView = currentWebTab->view();
+            webView->resize(currentView->size());
+            webView->page()->setViewportSize(currentView->page()->viewportSize());
+        }
+    }
+
+#ifdef Q_OS_WIN
+    QTimer::singleShot(0, p_QupZilla, SLOT(applyBlurToMainWindow()));
+#endif
     return index;
 }
 
+int TabWidget::addView(WebTab* tab)
+{
+    m_locationBars->addWidget(tab->locationBar());
+    tab->locationBar()->setWebView(tab->view());
+
+    int index = addTab(tab, QString());
+    setTabText(index, tab->title());
+
+    if (!tab->isLoading()) {
+        setTabIcon(index, tab->icon());
+    }
+    else {
+        startTabAnimation(index);
+    }
+
+    connect(tab->view(), SIGNAL(wantsCloseTab(int)), this, SLOT(closeTab(int)));
+    connect(tab->view(), SIGNAL(changed()), mApp, SLOT(setStateChanged()));
+    connect(tab->view(), SIGNAL(ipChanged(QString)), p_QupZilla->ipLabel(), SLOT(setText(QString)));
+
+    return index;
+}
+
+void TabWidget::addTabFromClipboard()
+{
+    QString selectionClipboard = QApplication::clipboard()->text(QClipboard::Selection);
+    QUrl guessedUrl = WebView::guessUrlFromString(selectionClipboard);
+
+    if (!guessedUrl.isEmpty()) {
+        addView(guessedUrl, Qz::NT_SelectedNewEmptyTab);
+    }
+}
+
 void TabWidget::closeTab(int index, bool force)
 {
     if (index == -1) {
@@ -372,16 +467,12 @@ void TabWidget::closeTab(int index, bool force)
         }
     }
 
-    if (webTab->isPinned()) {
-        emit pinnedTabClosed();
-    }
-
     m_locationBars->removeWidget(webView->webTab()->locationBar());
     disconnect(webView, SIGNAL(wantsCloseTab(int)), this, SLOT(closeTab(int)));
     disconnect(webView, SIGNAL(changed()), mApp, SLOT(setStateChanged()));
     disconnect(webView, SIGNAL(ipChanged(QString)), p_QupZilla->ipLabel(), SLOT(setText(QString)));
 
-    //Save last tab url and history
+    // Save last tab url and history
     m_closedTabsManager->saveView(webTab, index);
 
     if (m_isClosingToLastTabIndex && m_lastTabIndex < count() && index == currentIndex()) {
@@ -395,6 +486,11 @@ void TabWidget::closeTab(int index, bool force)
     webTab->disconnectObjects();
 
     webTab->deleteLater();
+
+    if (!m_closedInsteadOpened && m_menuTabs->isVisible()) {
+        QAction* labelAction = m_menuTabs->actions().last();
+        labelAction->setText(tr("Currently you have %n opened tab(s)", "", count() - 1));
+    }
 }
 
 void TabWidget::currentTabChanged(int index)
@@ -410,7 +506,7 @@ void TabWidget::currentTabChanged(int index)
     WebTab* webTab = weTab(index);
     LocationBar* locBar = webTab->locationBar();
 
-    if (m_locationBars->indexOf(locBar) != -1) {
+    if (locBar && m_locationBars->indexOf(locBar) != -1) {
         m_locationBars->setCurrentWidget(locBar);
     }
 
@@ -428,30 +524,17 @@ void TabWidget::tabMoved(int before, int after)
     m_lastTabIndex = before;
 }
 
-void TabWidget::tabInserted(int index)
-{
-    Q_UNUSED(index)
-
-    tabBar()->setVisible(!(count() == 1 && m_hideTabBarWithOneTab));
-}
-
-void TabWidget::tabRemoved(int index)
-{
-    Q_UNUSED(index)
-
-    tabBar()->setVisible(!(count() == 1 && m_hideTabBarWithOneTab));
-}
-
 void TabWidget::startTabAnimation(int index)
 {
     if (!validIndex(index)) {
         return;
     }
 
-    QLabel* label = qobject_cast<QLabel*>(m_tabBar->tabButton(index, QTabBar::LeftSide));
+    QLabel* label = qobject_cast<QLabel*>(m_tabBar->tabButton(index, m_tabBar->iconButtonPosition()));
     if (!label) {
         label = new QLabel();
-        m_tabBar->setTabButton(index, QTabBar::LeftSide, label);
+        label->setObjectName("tab-icon");
+        m_tabBar->setTabButton(index, m_tabBar->iconButtonPosition(), label);
     }
 
     if (label->movie()) {
@@ -460,7 +543,6 @@ void TabWidget::startTabAnimation(int index)
     }
 
     QMovie* movie = new QMovie(":icons/other/progress.gif", QByteArray(), label);
-    movie->setSpeed(70);
     movie->start();
 
     label->setMovie(movie);
@@ -472,7 +554,7 @@ void TabWidget::stopTabAnimation(int index)
         return;
     }
 
-    QLabel* label = qobject_cast<QLabel*>(m_tabBar->tabButton(index, QTabBar::LeftSide));
+    QLabel* label = qobject_cast<QLabel*>(m_tabBar->tabButton(index, m_tabBar->iconButtonPosition()));
 
     if (label && label->movie()) {
         label->movie()->stop();
@@ -483,7 +565,7 @@ void TabWidget::setCurrentIndex(int index)
 {
     m_lastTabIndex = currentIndex();
 
-    QTabWidget::setCurrentIndex(index);
+    TabStackedWidget::setCurrentIndex(index);
 }
 
 void TabWidget::setTabIcon(int index, const QIcon &icon)
@@ -492,11 +574,12 @@ void TabWidget::setTabIcon(int index, const QIcon &icon)
         return;
     }
 
-    QLabel* label = qobject_cast<QLabel*>(m_tabBar->tabButton(index, QTabBar::LeftSide));
+    QLabel* label = qobject_cast<QLabel*>(m_tabBar->tabButton(index, m_tabBar->iconButtonPosition()));
     if (!label) {
         label = new QLabel();
+        label->setObjectName("tab-icon");
         label->resize(16, 16);
-        m_tabBar->setTabButton(index, QTabBar::LeftSide, label);
+        m_tabBar->setTabButton(index, m_tabBar->iconButtonPosition(), label);
     }
 
     label->setPixmap(icon.pixmap(16, 16));
@@ -518,7 +601,7 @@ void TabWidget::setTabText(int index, const QString &text)
     }
 
     setTabToolTip(index, text);
-    QTabWidget::setTabText(index, newtext);
+    TabStackedWidget::setTabText(index, newtext);
 }
 
 void TabWidget::nextTab()
@@ -552,21 +635,21 @@ void TabWidget::reloadTab(int index)
     weTab(index)->reload();
 }
 
-void TabWidget::showTabBar()
-{
-    if (count() == 1 && m_hideTabBarWithOneTab) {
-        tabBar()->hide();
-    }
-    else {
-        tabBar()->show();
-    }
-}
-
 int TabWidget::lastTabIndex() const
 {
     return m_lastTabIndex;
 }
 
+TabBar* TabWidget::getTabBar() const
+{
+    return m_tabBar;
+}
+
+ClosedTabsManager* TabWidget::closedTabsManager() const
+{
+    return m_closedTabsManager;
+}
+
 void TabWidget::reloadAllTabs()
 {
     for (int i = 0; i < count(); i++) {
@@ -591,7 +674,7 @@ void TabWidget::closeAllButCurrent(int index)
 
     WebTab* akt = weTab(index);
 
-    foreach(WebTab * tab, allTabs(false)) {
+    foreach (WebTab* tab, allTabs(false)) {
         int tabIndex = tab->tabIndex();
         if (akt == widget(tabIndex)) {
             continue;
@@ -600,6 +683,28 @@ void TabWidget::closeAllButCurrent(int index)
     }
 }
 
+void TabWidget::detachTab(int index)
+{
+    WebTab* tab = weTab(index);
+
+    if (tab->isPinned() || count() == 1) {
+        return;
+    }
+
+    m_locationBars->removeWidget(tab->locationBar());
+    disconnect(tab->view(), SIGNAL(wantsCloseTab(int)), this, SLOT(closeTab(int)));
+    disconnect(tab->view(), SIGNAL(changed()), mApp, SLOT(setStateChanged()));
+    disconnect(tab->view(), SIGNAL(ipChanged(QString)), p_QupZilla->ipLabel(), SLOT(setText(QString)));
+
+    QupZilla* window = mApp->makeNewWindow(Qz::BW_NewWindow);
+    tab->moveToWindow(window);
+    window->openWithTab(tab);
+
+    if (m_isClosingToLastTabIndex && m_lastTabIndex < count() && index == currentIndex()) {
+        setCurrentIndex(m_lastTabIndex);
+    }
+}
+
 int TabWidget::duplicateTab(int index)
 {
     if (!validIndex(index)) {
@@ -608,9 +713,9 @@ int TabWidget::duplicateTab(int index)
 
     WebTab* webTab = weTab(index);
 
-    const QUrl &url = webTab->url();
-    const QString &title = webTab->title();
-    const QByteArray &history = webTab->historyData();
+    const QUrl url = webTab->url();
+    const QString title = webTab->title();
+    const QByteArray history = webTab->historyData();
 
     QNetworkRequest req(url);
     req.setRawHeader("Referer", url.toEncoded());
@@ -622,15 +727,18 @@ int TabWidget::duplicateTab(int index)
     return id;
 }
 
-void TabWidget::restoreClosedTab()
+void TabWidget::restoreClosedTab(QObject* obj)
 {
+    if (!obj) {
+        obj = sender();
+    }
     if (!m_closedTabsManager->isClosedTabAvailable()) {
         return;
     }
 
     ClosedTabsManager::Tab tab;
 
-    QAction* action = qobject_cast<QAction*>(sender());
+    QAction* action = qobject_cast<QAction*>(obj);
     if (action && action->data().toInt() != 0) {
         tab = m_closedTabsManager->getTabAt(action->data().toInt());
     }
@@ -649,9 +757,9 @@ void TabWidget::restoreAllClosedTabs()
         return;
     }
 
-    const QList<ClosedTabsManager::Tab> &closedTabs = m_closedTabsManager->allClosedTabs();
+    const QVector<ClosedTabsManager::Tab> &closedTabs = m_closedTabsManager->allClosedTabs();
 
-    foreach(const ClosedTabsManager::Tab & tab, closedTabs) {
+    foreach (const ClosedTabsManager::Tab &tab, closedTabs) {
         int index = addView(QUrl(), tab.title, Qz::NT_CleanSelectedTab);
         WebTab* webTab = weTab(index);
         webTab->p_restoreTab(tab.url, tab.history);
@@ -665,11 +773,26 @@ void TabWidget::clearClosedTabsList()
     m_closedTabsManager->clearList();
 }
 
-bool TabWidget::canRestoreTab()
+bool TabWidget::canRestoreTab() const
 {
     return m_closedTabsManager->isClosedTabAvailable();
 }
 
+QStackedWidget* TabWidget::locationBars() const
+{
+    return m_locationBars;
+}
+
+ToolButton* TabWidget::buttonListTabs() const
+{
+    return m_buttonListTabs;
+}
+
+AddTabButton* TabWidget::buttonAddTab() const
+{
+    return m_buttonAddTab;
+}
+
 void TabWidget::aboutToShowClosedTabsMenu()
 {
     if (!m_closedInsteadOpened) {
@@ -678,7 +801,7 @@ void TabWidget::aboutToShowClosedTabsMenu()
     else {
         m_menuTabs->clear();
         int i = 0;
-        foreach(const ClosedTabsManager::Tab & tab, closedTabsManager()->allClosedTabs()) {
+        foreach (const ClosedTabsManager::Tab &tab, closedTabsManager()->allClosedTabs()) {
             QString title = tab.title;
             if (title.length() > 40) {
                 title.truncate(40);
@@ -722,6 +845,8 @@ void TabWidget::savePinnedTabs()
     QByteArray data;
     QDataStream stream(&data, QIODevice::WriteOnly);
 
+    stream << Qz::sessionVersion;
+
     QStringList tabs;
     QList<QByteArray> tabsHistory;
     for (int i = 0; i < count(); ++i) {
@@ -759,6 +884,12 @@ void TabWidget::restorePinnedTabs()
         return;
     }
 
+    int version;
+    stream >> version;
+    if (version != Qz::sessionVersion) {
+        return;
+    }
+
     QStringList pinnedTabs;
     stream >> pinnedTabs;
     QList<QByteArray> tabHistory;
@@ -773,23 +904,21 @@ void TabWidget::restorePinnedTabs()
         int addedIndex;
 
         if (!historyState.isEmpty()) {
-            addedIndex = addView(QUrl(), Qz::NT_CleanSelectedTab);
+            addedIndex = addView(QUrl(), Qz::NT_CleanSelectedTab, false, true);
 
             weTab(addedIndex)->p_restoreTab(url, historyState);
         }
         else {
-            addedIndex = addView(url);
+            addedIndex = addView(url, tr("New tab"), Qz::NT_SelectedTab, false, -1, true);
         }
 
         WebTab* webTab = weTab(addedIndex);
 
         if (webTab) {
             webTab->setPinned(true);
-            emit pinnedTabAdded();
         }
 
         m_tabBar->updatePinnedTabCloseButton(addedIndex);
-//        m_tabBar->moveTab(addedIndex, i);
     }
 
     m_isRestoringState = false;
@@ -797,7 +926,7 @@ void TabWidget::restorePinnedTabs()
 
 QByteArray TabWidget::saveState()
 {
-    QList<WebTab::SavedTab> tabList;
+    QVector<WebTab::SavedTab> tabList;
 
     for (int i = 0; i < count(); ++i) {
         WebTab* webTab = weTab(i);
@@ -814,7 +943,7 @@ QByteArray TabWidget::saveState()
 
     stream << tabList.count();
 
-    foreach(const WebTab::SavedTab & tab, tabList) {
+    foreach (const WebTab::SavedTab &tab, tabList) {
         stream << tab;
     }
 
@@ -823,9 +952,16 @@ QByteArray TabWidget::saveState()
     return data;
 }
 
-bool TabWidget::restoreState(const QList<WebTab::SavedTab> &tabs, int currentTab)
+bool TabWidget::restoreState(const QVector<WebTab::SavedTab> &tabs, int currentTab)
 {
     m_isRestoringState = true;
+    setUpdatesEnabled(false);
+
+    Qz::BrowserWindow type = p_QupZilla->windowType();
+
+    if (type == Qz::BW_FirstAppWindow || type == Qz::BW_MacFirstWindow) {
+        restorePinnedTabs();
+    }
 
     for (int i = 0; i < tabs.size(); ++i) {
         WebTab::SavedTab tab = tabs.at(i);
@@ -835,6 +971,8 @@ bool TabWidget::restoreState(const QList<WebTab::SavedTab> &tabs, int currentTab
     }
 
     m_isRestoringState = false;
+    setUpdatesEnabled(true);
+
 
     setCurrentIndex(currentTab);
     currentTabChanged(currentTab);
@@ -844,7 +982,7 @@ bool TabWidget::restoreState(const QList<WebTab::SavedTab> &tabs, int currentTab
 
 void TabWidget::closeRecoveryTab()
 {
-    foreach(WebTab * tab, allTabs(false)) {
+    foreach (WebTab* tab, allTabs(false)) {
         if (tab->url().toString() == QLatin1String("qupzilla:restore")) {
             closeTab(tab->tabIndex(), true);
         }
