@@ -1,6 +1,6 @@
 /* ============================================================
 * QupZilla - WebKit based browser
-* Copyright (C) 2010-2012  David Rosca <nowrep@gmail.com>
+* Copyright (C) 2010-2014  David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -15,13 +15,14 @@
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ============================================================ */
+
 #include "webview.h"
 #include "webpage.h"
 #include "mainapplication.h"
-#include "globalfunctions.h"
+#include "qztools.h"
 #include "iconprovider.h"
 #include "history.h"
-#include "autofillmodel.h"
+#include "pluginproxy.h"
 #include "downloadmanager.h"
 #include "sourceviewer.h"
 #include "siteinfo.h"
@@ -31,7 +32,14 @@
 #include "settings.h"
 #include "qzsettings.h"
 #include "enhancedmenu.h"
-#include "pluginproxy.h"
+
+#ifdef USE_HUNSPELL
+#include "qtwebkit/spellcheck/speller.h"
+#endif
+
+#ifdef Q_OS_MAC
+#include "macwebviewscroller.h"
+#endif
 
 #include <QDir>
 #include <QTimer>
@@ -54,6 +62,9 @@ WebView::WebView(QWidget* parent)
     , m_actionStop(0)
     , m_actionsInitialized(false)
     , m_disableTouchMocking(false)
+    , m_isReloading(false)
+    , m_hasRss(false)
+    , m_rssChecked(false)
 {
     connect(this, SIGNAL(loadStarted()), this, SLOT(slotLoadStarted()));
     connect(this, SIGNAL(loadProgress(int)), this, SLOT(slotLoadProgress(int)));
@@ -64,7 +75,13 @@ WebView::WebView(QWidget* parent)
     // Zoom levels same as in firefox
     m_zoomLevels << 30 << 50 << 67 << 80 << 90 << 100 << 110 << 120 << 133 << 150 << 170 << 200 << 240 << 300;
 
+#if QTWEBKIT_TO_2_3
     installEventFilter(this);
+#endif
+
+#ifdef Q_OS_MAC
+    new MacWebViewScroller(this);
+#endif
 }
 
 QIcon WebView::icon() const
@@ -77,6 +94,10 @@ QIcon WebView::icon() const
         return qIconProvider->standardIcon(QStyle::SP_DriveHDIcon);
     }
 
+    if (url().scheme() == QLatin1String("ftp")) {
+        return qIconProvider->standardIcon(QStyle::SP_ComputerIcon);
+    }
+
     if (!QWebView::icon().isNull()) {
         return QWebView::icon();
     }
@@ -133,10 +154,16 @@ void WebView::setPage(QWebPage* page)
     m_page = qobject_cast<WebPage*>(page);
 
     setZoom(qzSettings->defaultZoom);
-    connect(m_page, SIGNAL(saveFrameStateRequested(QWebFrame*, QWebHistoryItem*)), this, SLOT(frameStateChanged()));
+    connect(m_page, SIGNAL(saveFrameStateRequested(QWebFrame*,QWebHistoryItem*)), this, SLOT(frameStateChanged()));
     connect(m_page, SIGNAL(privacyChanged(bool)), this, SIGNAL(privacyChanged(bool)));
 
     mApp->plugins()->emitWebPageCreated(m_page);
+
+    // Set white background by default.
+    // Fixes issue with dark themes. See #602
+    QPalette pal = palette();
+    pal.setBrush(QPalette::Base, Qt::white);
+    page->setPalette(pal);
 }
 
 void WebView::load(const QUrl &url)
@@ -146,11 +173,18 @@ void WebView::load(const QUrl &url)
 
 void WebView::load(const QNetworkRequest &request, QNetworkAccessManager::Operation operation, const QByteArray &body)
 {
-    const QUrl &reqUrl = request.url();
+    const QUrl reqUrl = request.url();
 
     if (reqUrl.scheme() == QLatin1String("javascript")) {
         // Getting scriptSource from PercentEncoding to properly load bookmarklets
-        QString scriptSource = QUrl::fromPercentEncoding(reqUrl.toString().mid(11).toUtf8());
+        // First check if url is percent encoded (let's just look for space)
+        QString scriptSource;
+        if (reqUrl.path().trimmed().contains(' ')) {
+            scriptSource = reqUrl.toString().mid(11);
+        }
+        else {
+            scriptSource = QUrl::fromPercentEncoding(reqUrl.toString().mid(11).toUtf8());
+        }
         page()->mainFrame()->evaluateJavaScript(scriptSource);
         return;
     }
@@ -162,8 +196,10 @@ void WebView::load(const QNetworkRequest &request, QNetworkAccessManager::Operat
         return;
     }
 
-    const QUrl &searchUrl = mApp->searchEnginesManager()->searchUrl(reqUrl.toString());
-    QWebView::load(searchUrl);
+    SearchEnginesManager::SearchResult res = mApp->searchEnginesManager()->searchResult(reqUrl.toString());
+    const QUrl searchUrl = res.request.url();
+
+    QWebView::load(res.request, res.operation, res.data);
 
     emit urlChanged(searchUrl);
     m_aboutToLoadUrl = searchUrl;
@@ -190,23 +226,21 @@ void WebView::fakeLoadingProgress(int progress)
     emit loadProgress(progress);
 }
 
+bool WebView::hasRss() const
+{
+    return m_hasRss;
+}
+
+QWebElement WebView::activeElement() const
+{
+    QRect activeRect = inputMethodQuery(Qt::ImMicroFocus).toRect();
+    return page()->mainFrame()->hitTestContent(activeRect.center()).element();
+}
+
 bool WebView::isUrlValid(const QUrl &url)
 {
-    const QString &urlScheme = url.scheme();
-    if (urlScheme == QLatin1String("data") || urlScheme == QLatin1String("qrc") ||
-            urlScheme == QLatin1String("mailto")) {
-        return true;
-    }
-
-    if (urlScheme == QLatin1String("qupzilla") || urlScheme == QLatin1String("file")) {
-        return !url.path().isEmpty();
-    }
-
-    if (url.isValid() && !url.host().isEmpty() && !urlScheme.isEmpty()) {
-        return true;
-    }
-
-    return false;
+    // Valid url must have scheme and actually contains something (therefore scheme:// is invalid)
+    return url.isValid() && !url.scheme().isEmpty() && (!url.host().isEmpty() || !url.path().isEmpty() || url.hasQuery());
 }
 
 QUrl WebView::guessUrlFromString(const QString &string)
@@ -284,6 +318,7 @@ void WebView::zoomReset()
 
 void WebView::reload()
 {
+    m_isReloading = true;
     if (QWebView::url().isEmpty() && !m_aboutToLoadUrl.isEmpty()) {
         load(m_aboutToLoadUrl);
         return;
@@ -316,6 +351,12 @@ void WebView::forward()
     }
 }
 
+void WebView::editDelete()
+{
+    QKeyEvent ev(QEvent::KeyPress, Qt::Key_Delete, Qt::NoModifier);
+    QApplication::sendEvent(this, &ev);
+}
+
 void WebView::selectAll()
 {
     triggerPageAction(QWebPage::SelectAll);
@@ -330,11 +371,18 @@ void WebView::slotLoadStarted()
         m_actionStop->setEnabled(true);
         m_actionReload->setEnabled(false);
     }
+
+    m_rssChecked = false;
+    emit rssChanged(false);
 }
 
 void WebView::slotLoadProgress(int progress)
 {
     m_progress = progress;
+
+    if (m_progress > 60) {
+        checkRss();
+    }
 }
 
 void WebView::slotLoadFinished()
@@ -347,12 +395,11 @@ void WebView::slotLoadFinished()
         m_actionReload->setEnabled(true);
     }
 
-    if (m_lastUrl != url()) {
+    if (!m_isReloading) {
         mApp->history()->addHistoryEntry(this);
     }
 
-    mApp->autoFill()->completePage(page());
-
+    m_isReloading = false;
     m_lastUrl = url();
 }
 
@@ -367,6 +414,20 @@ void WebView::emitChangedUrl()
     emit urlChanged(url());
 }
 
+void WebView::checkRss()
+{
+    if (m_rssChecked) {
+        return;
+    }
+
+    m_rssChecked = true;
+    QWebFrame* frame = page()->mainFrame();
+    const QWebElementCollection links = frame->findAllElements("link[type=\"application/rss+xml\"]");
+
+    m_hasRss = links.count() != 0;
+    emit rssChanged(m_hasRss);
+}
+
 void WebView::slotIconChanged()
 {
     if (!loadingError()) {
@@ -379,12 +440,19 @@ void WebView::slotIconChanged()
 
 void WebView::slotUrlChanged(const QUrl &url)
 {
-    // Disable touch mocking on all google pages as it just makes it buggy
-    if (url.host().contains(QLatin1String("google"))) {
-        m_disableTouchMocking = true;
+    static QStringList exceptions;
+    if (exceptions.isEmpty()) {
+        exceptions << "google." << "twitter.";
     }
-    else {
-        m_disableTouchMocking = false;
+
+    // Disable touch mocking on pages known not to work properly
+    const QString host = url.host();
+    m_disableTouchMocking = false;
+
+    foreach (const QString &site, exceptions) {
+        if (host.contains(site)) {
+            m_disableTouchMocking = true;
+        }
     }
 }
 
@@ -398,28 +466,32 @@ void WebView::openUrlInNewWindow()
 void WebView::sendLinkByMail()
 {
     if (QAction* action = qobject_cast<QAction*>(sender())) {
-        const QUrl &mailUrl = QUrl::fromEncoded("mailto:?body=" + QUrl::toPercentEncoding(action->data().toUrl().toEncoded()));
+        const QUrl mailUrl = QUrl::fromEncoded("mailto:%20?body=" + QUrl::toPercentEncoding(action->data().toUrl().toEncoded()));
         QDesktopServices::openUrl(mailUrl);
     }
 }
 
 void WebView::sendPageByMail()
 {
-    const QUrl &mailUrl = QUrl::fromEncoded("mailto:?body=" + QUrl::toPercentEncoding(url().toEncoded()) + "&subject=" + QUrl::toPercentEncoding(title()));
+    const QUrl mailUrl = QUrl::fromEncoded("mailto:%20?body=" + QUrl::toPercentEncoding(url().toEncoded()) + "&subject=" + QUrl::toPercentEncoding(title()));
     QDesktopServices::openUrl(mailUrl);
 }
 
 void WebView::copyLinkToClipboard()
 {
     if (QAction* action = qobject_cast<QAction*>(sender())) {
-        QApplication::clipboard()->setText(action->data().toString());
+        QApplication::clipboard()->setText(action->data().toUrl().toEncoded());
     }
 }
 
 void WebView::savePageAs()
 {
+    if (url().isEmpty() || url().toString() == QLatin1String("about:blank")) {
+        return;
+    }
+
     QNetworkRequest request(url());
-    QString suggestedFileName = qz_getFileNameFromUrl(url());
+    QString suggestedFileName = QzTools::getFileNameFromUrl(url());
     if (!suggestedFileName.contains(QLatin1Char('.'))) {
         suggestedFileName.append(QLatin1String(".html"));
     }
@@ -434,6 +506,11 @@ void WebView::savePageAs()
     dManager->download(request, info);
 }
 
+void WebView::openUrlInNewTab(const QUrl &url, Qz::NewTabPositionFlag position)
+{
+    loadInNewTab(QNetworkRequest(url), QNetworkAccessManager::GetOperation, QByteArray(), position);
+}
+
 void WebView::downloadUrlToDisk()
 {
     if (QAction* action = qobject_cast<QAction*>(sender())) {
@@ -469,7 +546,7 @@ void WebView::showSource(QWebFrame* frame, const QString &selectedHtml)
     }
 
     SourceViewer* source = new SourceViewer(frame, selectedHtml);
-    qz_centerWidgetToParent(source, this);
+    QzTools::centerWidgetToParent(source, this);
     source->show();
 }
 
@@ -484,12 +561,12 @@ void WebView::searchSelectedText()
     SearchEngine engine = mApp->searchEnginesManager()->activeEngine();
     if (QAction* act = qobject_cast<QAction*>(sender())) {
         if (act->data().isValid()) {
-            engine = qVariantValue<SearchEngine>(act->data());
+            engine = act->data().value<SearchEngine>();
         }
     }
 
-    const QUrl &urlToLoad = mApp->searchEnginesManager()->searchUrl(engine, selectedText());
-    openUrlInNewTab(urlToLoad, Qz::NT_SelectedTab);
+    SearchEnginesManager::SearchResult res = mApp->searchEnginesManager()->searchResult(engine, selectedText());
+    loadInNewTab(res.request, res.operation, res.data, Qz::NT_SelectedTab);
 }
 
 void WebView::searchSelectedTextInBackgroundTab()
@@ -497,12 +574,12 @@ void WebView::searchSelectedTextInBackgroundTab()
     SearchEngine engine = mApp->searchEnginesManager()->activeEngine();
     if (QAction* act = qobject_cast<QAction*>(sender())) {
         if (act->data().isValid()) {
-            engine = qVariantValue<SearchEngine>(act->data());
+            engine = act->data().value<SearchEngine>();
         }
     }
 
-    const QUrl &urlToLoad = mApp->searchEnginesManager()->searchUrl(engine, selectedText());
-    openUrlInNewTab(urlToLoad, Qz::NT_NotSelectedTab);
+    SearchEnginesManager::SearchResult res = mApp->searchEnginesManager()->searchResult(engine, selectedText());
+    loadInNewTab(res.request, res.operation, res.data, Qz::NT_NotSelectedTab);
 }
 
 void WebView::bookmarkLink()
@@ -512,14 +589,17 @@ void WebView::bookmarkLink()
             mApp->browsingLibrary()->bookmarksManager()->addBookmark(this);
         }
         else {
-            mApp->browsingLibrary()->bookmarksManager()->insertBookmark(action->data().toUrl(), title(), icon());
+            const QVariantList bData = action->data().value<QVariantList>();
+            const QString bookmarkTitle = bData.at(1).toString().isEmpty() ? title() : bData.at(1).toString();
+
+            mApp->browsingLibrary()->bookmarksManager()->insertBookmark(bData.at(0).toUrl(), bookmarkTitle, icon());
         }
     }
 }
 
 void WebView::showSourceOfSelection()
 {
-#if (QTWEBKIT_VERSION >= QTWEBKIT_VERSION_CHECK(2, 2, 0))
+#if QTWEBKIT_FROM_2_2
     showSource(page()->mainFrame(), selectedHtml());
 #endif
 }
@@ -545,12 +625,30 @@ void WebView::userDefinedOpenUrlInNewTab(const QUrl &url, bool invert)
         position = (position == Qz::NT_SelectedTab) ? Qz::NT_NotSelectedTab : Qz::NT_SelectedTab;
     }
 
-    if (QAction* action = qobject_cast<QAction*>(sender())) {
-        openUrlInNewTab(action->data().toUrl(), position);
+    QUrl actionUrl;
+
+    if (!url.isEmpty()) {
+        actionUrl = url;
     }
-    else {
-        openUrlInNewTab(url, position);
+    else if (QAction* action = qobject_cast<QAction*>(sender())) {
+        actionUrl = action->data().toUrl();
     }
+
+    openUrlInNewTab(actionUrl, position);
+}
+
+void WebView::userDefinedOpenUrlInBgTab(const QUrl &url)
+{
+    QUrl actionUrl;
+
+    if (!url.isEmpty()) {
+        actionUrl = url;
+    }
+    else if (QAction* action = qobject_cast<QAction*>(sender())) {
+        actionUrl = action->data().toUrl();
+    }
+
+    userDefinedOpenUrlInNewTab(actionUrl, true);
 }
 
 void WebView::loadClickedFrame()
@@ -563,14 +661,19 @@ void WebView::loadClickedFrame()
     load(frameUrl);
 }
 
-void WebView::loadClickedFrameInNewTab()
+void WebView::loadClickedFrameInNewTab(bool invert)
 {
     QUrl frameUrl = m_clickedFrame->baseUrl();
     if (frameUrl.isEmpty()) {
         frameUrl = m_clickedFrame->requestedUrl();
     }
 
-    openUrlInNewTab(frameUrl, Qz::NT_SelectedTab);
+    userDefinedOpenUrlInNewTab(frameUrl, invert);
+}
+
+void WebView::loadClickedFrameInBgTab()
+{
+    loadClickedFrameInNewTab(true);
 }
 
 void WebView::reloadClickedFrame()
@@ -661,10 +764,10 @@ void WebView::checkForForm(QMenu* menu, const QWebElement &element)
         return;
     }
 
-    const QString &url = parentElement.attribute("action");
-    const QString &method = parentElement.hasAttribute("method") ? parentElement.attribute("method").toUpper() : "GET";
+    const QString url = parentElement.attribute("action");
+    const QString method = parentElement.hasAttribute("method") ? parentElement.attribute("method").toUpper() : "GET";
 
-    if (!url.isEmpty() && method == QLatin1String("GET")) {
+    if (!url.isEmpty() && (method == QLatin1String("GET") || method == QLatin1String("POST"))) {
         menu->addAction(QIcon(":icons/menu/search-icon.png"), tr("Create Search Engine"), this, SLOT(createSearchEngine()));
 
         m_clickedElement = element;
@@ -682,9 +785,20 @@ void WebView::createContextMenu(QMenu* menu, const QWebHitTestResult &hitTest, c
         m_actionsInitialized = true;
 
         pageAction(QWebPage::Cut)->setIcon(QIcon::fromTheme("edit-cut"));
+        pageAction(QWebPage::Cut)->setText(tr("Cut"));
         pageAction(QWebPage::Copy)->setIcon(QIcon::fromTheme("edit-copy"));
+        pageAction(QWebPage::Copy)->setText(tr("Copy"));
         pageAction(QWebPage::Paste)->setIcon(QIcon::fromTheme("edit-paste"));
+        pageAction(QWebPage::Paste)->setText(tr("Paste"));
         pageAction(QWebPage::SelectAll)->setIcon(QIcon::fromTheme("edit-select-all"));
+        pageAction(QWebPage::SelectAll)->setText(tr("Select All"));
+
+        pageAction(QWebPage::SetTextDirectionDefault)->setText(tr("Default"));
+        pageAction(QWebPage::SetTextDirectionLeftToRight)->setText(tr("Left to Right"));
+        pageAction(QWebPage::SetTextDirectionRightToLeft)->setText(tr("Right to Left"));
+        pageAction(QWebPage::ToggleBold)->setText(tr("Bold"));
+        pageAction(QWebPage::ToggleItalic)->setText(tr("Italic"));
+        pageAction(QWebPage::ToggleUnderline)->setText(tr("Underline"));
 
         m_actionReload = new QAction(qIconProvider->standardIcon(QStyle::SP_BrowserReload), tr("&Reload"), this);
         m_actionStop = new QAction(qIconProvider->standardIcon(QStyle::SP_BrowserStop), tr("S&top"), this);
@@ -696,6 +810,17 @@ void WebView::createContextMenu(QMenu* menu, const QWebHitTestResult &hitTest, c
         m_actionStop->setEnabled(isLoading());
     }
 
+    // cppcheck-suppress variableScope
+    int spellCheckActionCount = 0;
+
+#ifdef USE_HUNSPELL
+    // Show spellcheck menu as the first
+    if (hitTest.isContentEditable() && !hitTest.isContentSelected()) {
+        mApp->speller()->populateContextMenu(menu, hitTest);
+        spellCheckActionCount = menu->actions().count();
+    }
+#endif
+
     if (!hitTest.linkUrl().isEmpty() && hitTest.linkUrl().scheme() != QLatin1String("javascript")) {
         createLinkContextMenu(menu, hitTest);
     }
@@ -709,40 +834,59 @@ void WebView::createContextMenu(QMenu* menu, const QWebHitTestResult &hitTest, c
     }
 
     if (hitTest.isContentEditable()) {
-        if (menu->isEmpty()) {
+        // This only checks if the menu is empty (only spellchecker actions added)
+        if (menu->actions().count() == spellCheckActionCount) {
             QMenu* pageMenu = page()->createStandardContextMenu();
-
-            int i = 0;
-            foreach(QAction * act, pageMenu->actions()) {
-                if (act->isSeparator()) {
+            // Apparently createStandardContextMenu() can return null pointer
+            if (pageMenu) {
+                if (qzSettings->enableFormsUndoRedo) {
+                    pageAction(QWebPage::Undo)->setIcon(QIcon::fromTheme("edit-undo"));
+                    pageAction(QWebPage::Undo)->setText(tr("Undo"));
+                    menu->addAction(pageAction(QWebPage::Undo));
+                    pageAction(QWebPage::Redo)->setIcon(QIcon::fromTheme("edit-redo"));
+                    pageAction(QWebPage::Redo)->setText(tr("Redo"));
+                    menu->addAction(pageAction(QWebPage::Redo));
                     menu->addSeparator();
-                    continue;
                 }
-
-                // Hiding double Direction + Fonts menu (bug in QtWebKit 2.2)
-                if (i <= 1 && act->menu()) {
-                    if (act->menu()->actions().contains(pageAction(QWebPage::SetTextDirectionDefault)) ||
-                            act->menu()->actions().contains(pageAction(QWebPage::ToggleBold))) {
-                        act->setVisible(false);
+                int i = 0;
+                foreach (QAction* act, pageMenu->actions()) {
+                    if (act->isSeparator()) {
+                        menu->addSeparator();
+                        continue;
                     }
+
+                    // Hiding double Direction + Fonts menu (bug in QtWebKit 2.2)
+                    if (i <= 1 && act->menu()) {
+                        if (act->menu()->actions().contains(pageAction(QWebPage::SetTextDirectionDefault)) ||
+                                act->menu()->actions().contains(pageAction(QWebPage::ToggleBold))) {
+                            act->setVisible(false);
+                        }
+                    }
+
+                    menu->addAction(act);
+
+                    if (act == pageAction(QWebPage::Paste)) {
+                        QAction* a = menu->addAction(QIcon::fromTheme("edit-delete"), tr("Delete"), this, SLOT(editDelete()));
+                        a->setEnabled(!selectedText().isEmpty());
+                    }
+
+                    ++i;
                 }
 
-                menu->addAction(act);
+                if (menu->actions().last() == pageAction(QWebPage::InspectElement)) {
+                    // We have own Inspect Element action
+                    menu->actions().last()->setVisible(false);
+                }
 
-                ++i;
+                delete pageMenu;
             }
-
-            if (menu->actions().last() == pageAction(QWebPage::InspectElement)) {
-                // We have own Inspect Element action
-                menu->actions().last()->setVisible(false);
-            }
-
-            delete pageMenu;
         }
 
         if (hitTest.element().tagName().toLower() == QLatin1String("input")) {
             checkForForm(menu, hitTest.element());
         }
+
+        createSpellCheckContextMenu(menu);
     }
 
     if (!selectedText().isEmpty()) {
@@ -756,8 +900,7 @@ void WebView::createContextMenu(QMenu* menu, const QWebHitTestResult &hitTest, c
     menu->addSeparator();
     mApp->plugins()->populateWebViewMenu(menu, this, hitTest);
 
-
-#if (QTWEBKIT_VERSION >= QTWEBKIT_VERSION_CHECK(2, 2, 0))
+#if QTWEBKIT_FROM_2_2
     //    still bugged? in 4.8 RC (it shows selection of webkit's internal source, not html from page)
     //    it may or may not be bug, but this implementation is useless for us
     //
@@ -778,45 +921,62 @@ void WebView::createPageContextMenu(QMenu* menu, const QPoint &pos)
     action->setIcon(qIconProvider->standardIcon(QStyle::SP_ArrowForward));
     action->setEnabled(history()->canGoForward());
 
-    menu->addAction(m_actionReload);
-    menu->addAction(m_actionStop);
-    menu->addSeparator();
+    if (url() != QUrl("qupzilla:speeddial")) {
 
-    if (frameAtPos && page()->mainFrame() != frameAtPos) {
-        m_clickedFrame = frameAtPos;
-        QMenu* frameMenu = new QMenu(tr("This frame"));
-        frameMenu->addAction(tr("Show &only this frame"), this, SLOT(loadClickedFrame()));
-        frameMenu->addAction(QIcon(":/icons/menu/popup.png"), tr("Show this frame in new &tab"), this, SLOT(loadClickedFrameInNewTab()));
-        frameMenu->addSeparator();
-        frameMenu->addAction(qIconProvider->standardIcon(QStyle::SP_BrowserReload), tr("&Reload"), this, SLOT(reloadClickedFrame()));
-        frameMenu->addAction(QIcon::fromTheme("document-print"), tr("Print frame"), this, SLOT(printClickedFrame()));
-        frameMenu->addSeparator();
-        frameMenu->addAction(QIcon::fromTheme("zoom-in"), tr("Zoom &in"), this, SLOT(clickedFrameZoomIn()));
-        frameMenu->addAction(QIcon::fromTheme("zoom-out"), tr("&Zoom out"), this, SLOT(clickedFrameZoomOut()));
-        frameMenu->addAction(QIcon::fromTheme("zoom-original"), tr("Reset"), this, SLOT(clickedFrameZoomReset()));
-        frameMenu->addSeparator();
-        frameMenu->addAction(QIcon::fromTheme("text-html"), tr("Show so&urce of frame"), this, SLOT(showClickedFrameSource()));
+        menu->addAction(m_actionReload);
+        menu->addAction(m_actionStop);
+        menu->addSeparator();
 
-        menu->addMenu(frameMenu);
+        if (frameAtPos && page()->mainFrame() != frameAtPos) {
+            m_clickedFrame = frameAtPos;
+            Menu* frameMenu = new Menu(tr("This frame"));
+            frameMenu->addAction(tr("Show &only this frame"), this, SLOT(loadClickedFrame()));
+            Action* act = new Action(QIcon::fromTheme("tab-new", QIcon(":/icons/menu/tab-new.png")), tr("Show this frame in new &tab"));
+            connect(act, SIGNAL(triggered()), this, SLOT(loadClickedFrameInNewTab()));
+            connect(act, SIGNAL(middleClicked()), this, SLOT(loadClickedFrameInBgTab()));
+            frameMenu->addAction(act);
+            frameMenu->addSeparator();
+            frameMenu->addAction(qIconProvider->standardIcon(QStyle::SP_BrowserReload), tr("&Reload"), this, SLOT(reloadClickedFrame()));
+            frameMenu->addAction(QIcon::fromTheme("document-print"), tr("Print frame"), this, SLOT(printClickedFrame()));
+            frameMenu->addSeparator();
+            frameMenu->addAction(QIcon::fromTheme("zoom-in"), tr("Zoom &in"), this, SLOT(clickedFrameZoomIn()));
+            frameMenu->addAction(QIcon::fromTheme("zoom-out"), tr("&Zoom out"), this, SLOT(clickedFrameZoomOut()));
+            frameMenu->addAction(QIcon::fromTheme("zoom-original"), tr("Reset"), this, SLOT(clickedFrameZoomReset()));
+            frameMenu->addSeparator();
+            frameMenu->addAction(QIcon::fromTheme("text-html"), tr("Show so&urce of frame"), this, SLOT(showClickedFrameSource()));
+
+            menu->addMenu(frameMenu);
+        }
+
+        menu->addSeparator();
+        menu->addAction(qIconProvider->fromTheme("bookmark-new"), tr("Book&mark page"), this, SLOT(bookmarkLink()));
+        menu->addAction(QIcon::fromTheme("document-save"), tr("&Save page as..."), this, SLOT(savePageAs()));
+        menu->addAction(QIcon::fromTheme("edit-copy"), tr("&Copy page link"), this, SLOT(copyLinkToClipboard()))->setData(url());
+        menu->addAction(QIcon::fromTheme("mail-message-new"), tr("Send page link..."), this, SLOT(sendPageByMail()));
+        menu->addAction(QIcon::fromTheme("document-print"), tr("&Print page"), this, SLOT(printPage()));
+        menu->addSeparator();
+        menu->addAction(QIcon::fromTheme("edit-select-all"), tr("Select &all"), this, SLOT(selectAll()));
+        menu->addSeparator();
+
+        if (url().scheme() == QLatin1String("http") || url().scheme() == QLatin1String("https")) {
+            const QUrl w3url = QUrl::fromEncoded("http://validator.w3.org/check?uri=" + QUrl::toPercentEncoding(url().toEncoded()));
+            menu->addAction(QIcon(":icons/sites/w3.png"), tr("Validate page"), this, SLOT(openUrlInSelectedTab()))->setData(w3url);
+
+            QByteArray langCode = mApp->currentLanguage().left(2).toUtf8();
+            const QUrl gturl = QUrl::fromEncoded("http://translate.google.com/translate?sl=auto&tl=" + langCode + "&u=" + QUrl::toPercentEncoding(url().toEncoded()));
+            menu->addAction(QIcon(":icons/sites/translate.png"), tr("Translate page"), this, SLOT(openUrlInSelectedTab()))->setData(gturl);
+        }
+
+        menu->addSeparator();
+        menu->addAction(QIcon::fromTheme("text-html"), tr("Show so&urce code"), this, SLOT(showSource()));
+        menu->addAction(QIcon::fromTheme("dialog-information"), tr("Show info ab&out site"), this, SLOT(showSiteInfo()));
     }
 
-    menu->addSeparator();
-    menu->addAction(qIconProvider->fromTheme("user-bookmarks"), tr("Book&mark page"), this, SLOT(bookmarkLink()));
-    menu->addAction(QIcon::fromTheme("document-save"), tr("&Save page as..."), this, SLOT(savePageAs()));
-    menu->addAction(QIcon::fromTheme("edit-copy"), tr("&Copy page link"), this, SLOT(copyLinkToClipboard()))->setData(url());
-    menu->addAction(QIcon::fromTheme("mail-message-new"), tr("Send page link..."), this, SLOT(sendPageByMail()));
-    menu->addAction(QIcon::fromTheme("document-print"), tr("&Print page"), this, SLOT(printPage()));
-    menu->addSeparator();
-    menu->addAction(QIcon::fromTheme("edit-select-all"), tr("Select &all"), this, SLOT(selectAll()));
-    menu->addSeparator();
-    if (url().scheme() == QLatin1String("http") || url().scheme() == QLatin1String("https")) {
-//        bool result = validateConfirm(tr("Do you want to upload this page to an online source code validator?"));
-//        if (result)
-        menu->addAction(tr("Validate page"), this, SLOT(openUrlInSelectedTab()))->setData(QUrl("http://validator.w3.org/check?uri=" + url().toString()));
+    else {
+        menu->addSeparator();
+        menu->addAction(QIcon::fromTheme("list-add"), tr("&Add New Page"), this, SLOT(addSpeedDial()));
+        menu->addAction(QIcon::fromTheme("configure"), tr("&Configure Speed Dial"), this, SLOT(configureSpeedDial()));
     }
-
-    menu->addAction(QIcon::fromTheme("text-html"), tr("Show so&urce code"), this, SLOT(showSource()));
-    menu->addAction(QIcon::fromTheme("dialog-information"), tr("Show info ab&out site"), this, SLOT(showSiteInfo()));
 }
 
 void WebView::createLinkContextMenu(QMenu* menu, const QWebHitTestResult &hitTest)
@@ -827,10 +987,18 @@ void WebView::createLinkContextMenu(QMenu* menu, const QWebHitTestResult &hitTes
     }
 
     menu->addSeparator();
-    menu->addAction(QIcon(":/icons/menu/popup.png"), tr("Open link in new &tab"), this, SLOT(userDefinedOpenUrlInNewTab()))->setData(hitTest.linkUrl());
+    Action* act = new Action(QIcon::fromTheme("tab-new", QIcon(":/icons/menu/tab-new.png")), tr("Open link in new &tab"));
+    act->setData(hitTest.linkUrl());
+    connect(act, SIGNAL(triggered()), this, SLOT(userDefinedOpenUrlInNewTab()));
+    connect(act, SIGNAL(middleClicked()), this, SLOT(userDefinedOpenUrlInBgTab()));
+    menu->addAction(act);
     menu->addAction(QIcon::fromTheme("window-new"), tr("Open link in new &window"), this, SLOT(openUrlInNewWindow()))->setData(hitTest.linkUrl());
     menu->addSeparator();
-    menu->addAction(qIconProvider->fromTheme("user-bookmarks"), tr("B&ookmark link"), this, SLOT(bookmarkLink()))->setData(hitTest.linkUrl());
+
+    QVariantList bData;
+    bData << hitTest.linkUrl() << hitTest.linkTitle();
+    menu->addAction(qIconProvider->fromTheme("bookmark-new"), tr("B&ookmark link"), this, SLOT(bookmarkLink()))->setData(bData);
+
     menu->addAction(QIcon::fromTheme("document-save"), tr("&Save link as..."), this, SLOT(downloadUrlToDisk()))->setData(hitTest.linkUrl());
     menu->addAction(QIcon::fromTheme("mail-message-new"), tr("Send link..."), this, SLOT(sendLinkByMail()))->setData(hitTest.linkUrl());
     menu->addAction(QIcon::fromTheme("edit-copy"), tr("&Copy link address"), this, SLOT(copyLinkToClipboard()))->setData(hitTest.linkUrl());
@@ -876,13 +1044,14 @@ void WebView::createSelectedTextContextMenu(QMenu* menu, const QWebHitTestResult
     menu->addAction(QIcon::fromTheme("mail-message-new"), tr("Send text..."), this, SLOT(sendLinkByMail()))->setData(selectedText);
     menu->addSeparator();
 
-    QString langCode = mApp->currentLanguage().left(2);
-    QUrl googleTranslateUrl = QUrl(QString("http://translate.google.com/#auto|%1|%2").arg(langCode, selectedText));
+    QString langCode = mApp->currentLanguage().left(2).toUtf8();
+    QUrl googleTranslateUrl = QUrl(QString("http://translate.google.com/#auto/%1/%2").arg(langCode, selectedText));
     Action* gtwact = new Action(QIcon(":icons/sites/translate.png"), tr("Google Translate"));
     gtwact->setData(googleTranslateUrl);
     connect(gtwact, SIGNAL(triggered()), this, SLOT(openUrlInSelectedTab()));
     connect(gtwact, SIGNAL(middleClicked()), this, SLOT(openUrlInBackgroundTab()));
     menu->addAction(gtwact);
+
     Action* dictact = new Action(QIcon::fromTheme("accessories-dictionary"), tr("Dictionary"));
     dictact->setData(QUrl("http://" + (!langCode.isEmpty() ? langCode + "." : langCode) + "wiktionary.org/wiki/Special:Search?search=" + selectedText));
     connect(dictact, SIGNAL(triggered()), this, SLOT(openUrlInSelectedTab()));
@@ -908,8 +1077,8 @@ void WebView::createSelectedTextContextMenu(QMenu* menu, const QWebHitTestResult
 
     menu->addSeparator();
     selectedText.truncate(20);
-    // KDE is displaying new lines in menu actions ... weird -,-
-    selectedText.replace(QLatin1Char('\n'), QLatin1Char(' ')).remove(QLatin1Char('\t'));
+    // KDE is displaying newlines in menu actions ... weird -,-
+    selectedText.replace(QLatin1Char('\n'), QLatin1Char(' ')).replace(QLatin1Char('\t'), QLatin1Char(' '));
 
     SearchEngine engine = mApp->searchEnginesManager()->activeEngine();
     Action* act = new Action(engine.icon, tr("Search \"%1 ..\" with %2").arg(selectedText, engine.name));
@@ -920,9 +1089,9 @@ void WebView::createSelectedTextContextMenu(QMenu* menu, const QWebHitTestResult
     // Search with ...
     Menu* swMenu = new Menu(tr("Search with..."), menu);
     SearchEnginesManager* searchManager = mApp->searchEnginesManager();
-    foreach(const SearchEngine & en, searchManager->allEngines()) {
+    foreach (const SearchEngine &en, searchManager->allEngines()) {
         Action* act = new Action(en.icon, en.name);
-        act->setData(qVariantFromValue(en));
+        act->setData(QVariant::fromValue(en));
 
         connect(act, SIGNAL(triggered()), this, SLOT(searchSelectedText()));
         connect(act, SIGNAL(middleClicked()), this, SLOT(searchSelectedTextInBackgroundTab()));
@@ -953,6 +1122,25 @@ void WebView::createMediaContextMenu(QMenu* menu, const QWebHitTestResult &hitTe
     menu->addAction(QIcon::fromTheme("document-save"), tr("Save Media To &Disk"), this, SLOT(downloadUrlToDisk()))->setData(videoUrl);
 }
 
+void WebView::createSpellCheckContextMenu(QMenu* menu)
+{
+    Q_UNUSED(menu)
+#ifdef USE_HUNSPELL
+    menu->addSeparator();
+
+    QAction* act = menu->addAction(tr("Check &Spelling"), mApp->speller(), SLOT(toggleEnableSpellChecking()));
+    act->setCheckable(true);
+    act->setChecked(mApp->speller()->isEnabled());
+
+    if (mApp->speller()->isEnabled()) {
+        QMenu* men = menu->addMenu(tr("Languages"));
+        connect(men, SIGNAL(aboutToShow()), mApp->speller(), SLOT(populateLanguagesMenu()));
+    }
+
+    menu->addSeparator();
+#endif
+}
+
 void WebView::pauseMedia()
 {
     bool paused = m_clickedElement.evaluateJavaScript("this.paused").toBool();
@@ -977,6 +1165,16 @@ void WebView::muteMedia()
     }
 }
 
+void WebView::addSpeedDial()
+{
+    page()->mainFrame()->evaluateJavaScript("addSpeedDial()");
+}
+
+void WebView::configureSpeedDial()
+{
+    page()->mainFrame()->evaluateJavaScript("configureSpeedDial()");
+}
+
 void WebView::wheelEvent(QWheelEvent* event)
 {
     if (mApp->plugins()->processWheelEvent(Qz::ON_WebView, this, event)) {
@@ -1025,7 +1223,7 @@ void WebView::mousePressEvent(QMouseEvent* event)
     case Qt::LeftButton: {
         QWebFrame* frame = page()->frameAt(event->pos());
         if (frame) {
-            const QUrl &link = frame->hitTestContent(event->pos()).linkUrl();
+            const QUrl link = frame->hitTestContent(event->pos()).linkUrl();
             if (event->modifiers() & Qt::ControlModifier && isUrlValid(link)) {
                 userDefinedOpenUrlInNewTab(link, event->modifiers() & Qt::ShiftModifier);
                 event->accept();
@@ -1051,7 +1249,7 @@ void WebView::mouseReleaseEvent(QMouseEvent* event)
     case Qt::MiddleButton: {
         QWebFrame* frame = page()->frameAt(event->pos());
         if (frame) {
-            const QUrl &link = frame->hitTestContent(event->pos()).linkUrl();
+            const QUrl link = frame->hitTestContent(event->pos()).linkUrl();
             if (m_clickedUrl == link && isUrlValid(link)) {
                 userDefinedOpenUrlInNewTab(link, event->modifiers() & Qt::ShiftModifier);
                 event->accept();
@@ -1084,7 +1282,26 @@ void WebView::keyPressEvent(QKeyEvent* event)
         return;
     }
 
-    switch (event->key()) {
+    int eventKey = event->key();
+
+    // The right/left arrow keys within contents with right to left (RTL) layout have
+    // reversed behavior than left to right (LTR) layout.
+    // Example: Key_Right within LTR layout triggers QWebPage::MoveToNextChar but,
+    // Key_Right within RTL layout should trigger QWebPage::MoveToPreviousChar
+
+    if (eventKey == Qt::Key_Left || eventKey == Qt::Key_Right) {
+        const QWebElement elementHasCursor = activeElement();
+        if (!elementHasCursor.isNull()) {
+            const QString direction = elementHasCursor.styleProperty("direction", QWebElement::ComputedStyle);
+            if (direction == QLatin1String("rtl")) {
+                eventKey = eventKey == Qt::Key_Left ? Qt::Key_Right : Qt::Key_Left;
+                QKeyEvent ev(event->type(), eventKey, event->modifiers(), event->text(), event->isAutoRepeat());
+                event = &ev;
+            }
+        }
+    }
+
+    switch (eventKey) {
     case Qt::Key_C:
         if (event->modifiers() == Qt::ControlModifier) {
             triggerPageAction(QWebPage::Copy);
@@ -1101,6 +1318,87 @@ void WebView::keyPressEvent(QKeyEvent* event)
         }
         break;
 
+    case Qt::Key_Up:
+        if (event->modifiers() & Qt::ShiftModifier) {
+            triggerPageAction(QWebPage::SelectPreviousLine);
+            event->accept();
+            return;
+        }
+        break;
+
+    case Qt::Key_Down:
+        if (event->modifiers() & Qt::ShiftModifier) {
+            triggerPageAction(QWebPage::SelectNextLine);
+            event->accept();
+            return;
+        }
+        break;
+
+    case Qt::Key_Left:
+        if (event->modifiers() & Qt::ShiftModifier) {
+            if (event->modifiers() == Qt::ShiftModifier) {
+                triggerPageAction(QWebPage::SelectPreviousChar);
+            }
+            else if (event->modifiers() == (Qt::ShiftModifier | Qt::ControlModifier)) {
+                triggerPageAction(QWebPage::SelectPreviousWord);
+            }
+            event->accept();
+            return;
+        }
+        break;
+
+    case Qt::Key_Right:
+        if (event->modifiers() & Qt::ShiftModifier) {
+            if (event->modifiers() == Qt::ShiftModifier) {
+                triggerPageAction(QWebPage::SelectNextChar);
+            }
+            else if (event->modifiers() == (Qt::ShiftModifier | Qt::ControlModifier)) {
+                triggerPageAction(QWebPage::SelectNextWord);
+            }
+            event->accept();
+            return;
+        }
+        break;
+
+    case Qt::Key_Home:
+        if (event->modifiers() & Qt::ShiftModifier) {
+            if (event->modifiers() == Qt::ShiftModifier) {
+                triggerPageAction(QWebPage::SelectStartOfLine);
+            }
+            else if (event->modifiers() == (Qt::ShiftModifier | Qt::ControlModifier)) {
+                triggerPageAction(QWebPage::SelectStartOfDocument);
+            }
+            event->accept();
+            return;
+        }
+        break;
+
+    case Qt::Key_End:
+        if (event->modifiers() & Qt::ShiftModifier) {
+            if (event->modifiers() == Qt::ShiftModifier) {
+                triggerPageAction(QWebPage::SelectEndOfLine);
+            }
+            else if (event->modifiers() == (Qt::ShiftModifier | Qt::ControlModifier)) {
+                triggerPageAction(QWebPage::SelectEndOfDocument);
+            }
+            event->accept();
+            return;
+        }
+        break;
+
+    case Qt::Key_Insert:
+        if (event->modifiers() == Qt::ControlModifier) {
+            triggerPageAction(QWebPage::Copy);
+            event->accept();
+            return;
+        }
+        if (event->modifiers() == Qt::ShiftModifier) {
+            triggerPageAction(QWebPage::Paste);
+            event->accept();
+            return;
+        }
+        break;
+
     default:
         break;
     }
@@ -1136,6 +1434,8 @@ void WebView::setZoom(int zoom)
 ///
 bool WebView::eventFilter(QObject* obj, QEvent* event)
 {
+// This hack is no longer needed with QtWebKit 2.3 (bundled in Qt 5)
+#if QTWEBKIT_TO_2_3
     if (obj != this || m_disableTouchMocking) {
         return false;
     }
@@ -1195,7 +1495,7 @@ bool WebView::eventFilter(QObject* obj, QEvent* event)
 
         return false;
     }
-
+#endif
     return QWebView::eventFilter(obj, event);
 }
 
